﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Runtime.InteropServices;
using GMap.NET;
using GMap.NET.MapProviders;
using GMap.NET.WindowsForms;
using GMap.NET.WindowsForms.Markers;
using GMap.NET.WindowsForms.ToolTips;
using System.Diagnostics;
//using System.ComponentModel;


namespace ShapeFileConverter
{

    public partial class MainForm : Form
    {
        private const UInt16 MAXN = 4860;

        private Int32 gs32ColumnNumber;

        private Byte[] gau8ShapeType = { 0, 1, 3, 5, 8, 11, 13, 15, 18, 21, 23, 25, 28, 31 };
        private String[] gasShapeType = { "Null Shape", "Point", "PolyLine", "Polygon", "MultiPoint", "PointZ", "PolyLineZ", "PolygonZ", "MultiPointZ", "PointM", "PolyLineM", "PolygonM", "MultilineM", "Multipatch" };
        private stPOLYGON[] gastPolygon = new stPOLYGON[100];
        private stDBASE_DATA gstDBase = new stDBASE_DATA();
        private stRECORDS gstRecords = new stRECORDS();
        private PointLatLng gstMinPoint = new PointLatLng();
        private PointLatLng gstMaxPoint = new PointLatLng();
        private GMapOverlay goMainOverlay = new GMapOverlay("polygons");
        private GMapOverlay goMarkersOverlay = new GMapOverlay("markers");
        private GMapOverlay goTempMarkerOverlay = new GMapOverlay("TempMarkerOverlay");
        private eSPRAY_STATE[,] gau8Field = new eSPRAY_STATE[MAXN, MAXN];
        public DBaseForm fDBaseForm;

        private delegate void MarshalToForm(String sComponent, String sActionText);

        public MainForm()
        {
            InitializeComponent();

            GMapControl.Overlays.Add(goMarkersOverlay);
            GMapControl.Overlays.Add(goMainOverlay);
            GMapControl.Overlays.Add(goTempMarkerOverlay);

            fnMainInit();
        }

        private void fnMainInit()
        {
            gstMaxPoint.Lng = 0;
            gstMaxPoint.Lat = 0;
            gstMinPoint.Lng = 0;
            gstMinPoint.Lat = 0;
            GPSPointsDataGridView.Rows.Clear();
            GPSPointsDataGridView.RowCount = 1;

            gstRecords.s32NumberOfEntries = 0;
            gs32NextRecord = 0;

            gflUpdate = false;
            MapsComboBox.Text = "Google Maps";
            gflUpdate = true;

            goMainOverlay.Clear();
            goMarkersOverlay.Clear();

            gstRecords.as32RecordNumber = new Int32[100];
            gstRecords.aeShapeType = new eShapeType[100];
            gstRecords.adbArea = new Double[100];
            for (Int32 i = 0; i < 100; i++)
            {
                gastPolygon[i].as32Position = new Int32[5];
                gastPolygon[i].astPoints = new List<PointLatLng>();
            }

            fDBaseForm = new DBaseForm();

            fDBaseForm.fnUpdateForm = new DBaseForm.deUpdateForm(fnMarshalToForm);

            PlusButton.BackColor = Color.Transparent;
            MinusButton.BackColor = Color.Transparent;
        }

        private void fnMarshalToForm(String sComponent, String sActionText)
        {
            object[] args = { sComponent, sActionText };

            MarshalToForm MarshalToFormDelegate = new MarshalToForm(fnUpdateForm);

            base.Invoke(MarshalToFormDelegate, args);
        }

        private void fnUpdateForm(String sComponent, String sActionText)
        {
            switch (sComponent)
            {
                case "Grower Information":
                    GMapControl.Enabled = true;

                    if (DBaseDataGridView.Columns.Count == gs32ColumnNumber + 1)
                    {
                        DBaseDataGridView.Columns.Add("", "");
                        DBaseDataGridView.Columns[gs32ColumnNumber].SortMode = DataGridViewColumnSortMode.NotSortable;
                    }

                    DBaseDataGridView.Columns[gs32ColumnNumber].HeaderText = sActionText;
                    break;
            }
        }

        private void fnConvertToShapeFile()
        {
            FileStream oOutput;
            FileStream oInput;
            BinaryWriter oWriter;
            BinaryReader oReader;

            FileStream oFileSHX;
            BinaryWriter oWriteSHX;

            if (File.Exists("TempSHX.txt") == true)
                File.Delete("TempSHX.txt");
            oFileSHX = new FileStream("TempSHX.txt", FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite);
            oWriteSHX = new BinaryWriter(oFileSHX);

            if (File.Exists("TempFile.txt") == true)
                File.Delete("TempFile.txt");
            oOutput = new FileStream("TempFile.txt", FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite);
            oWriter = new BinaryWriter(oOutput);

            // File Header
            Byte[] au8Header = new byte[10000];
            Byte[] au8Data;

            DoubleUnion u64Temp = new DoubleUnion();
            u64Temp.u32_0 = 9994;
            au8Header[0] = u64Temp.u8_3;
            au8Header[1] = u64Temp.u8_2;
            au8Header[2] = u64Temp.u8_1;
            au8Header[3] = u64Temp.u8_0;

            u64Temp.u32_0 = 1000;
            au8Header[28] = u64Temp.u8_0;
            au8Header[29] = u64Temp.u8_1;
            au8Header[30] = u64Temp.u8_2;
            au8Header[31] = u64Temp.u8_3;

            au8Header[32] = (Byte)eShapeType.POLYGON;

            PointLatLng stMinPoint = new PointLatLng(100000, 100000);
            PointLatLng stMaxPoint = new PointLatLng(-100000, -100000);

            for (Int32 i = 0; i < gstRecords.s32NumberOfEntries; i++)
            {
                if (gastPolygon[i].stBoundingBox.stMinPoint == null)
                    continue;

                if (gastPolygon[i].stBoundingBox.stMinPoint.Lng < stMinPoint.Lng)
                    stMinPoint.Lng = gastPolygon[i].stBoundingBox.stMinPoint.Lng;
                if (gastPolygon[i].stBoundingBox.stMinPoint.Lat < stMinPoint.Lat)
                    stMinPoint.Lat = gastPolygon[i].stBoundingBox.stMinPoint.Lat;
                if (gastPolygon[i].stBoundingBox.stMaxPoint.Lng > stMaxPoint.Lng)
                    stMaxPoint.Lng = gastPolygon[i].stBoundingBox.stMaxPoint.Lng;
                if (gastPolygon[i].stBoundingBox.stMaxPoint.Lat < stMaxPoint.Lat)
                    stMaxPoint.Lat = gastPolygon[i].stBoundingBox.stMaxPoint.Lat;
            }

            u64Temp.db = stMinPoint.Lng;
            au8Header[36] = u64Temp.u8_0;
            au8Header[37] = u64Temp.u8_1;
            au8Header[38] = u64Temp.u8_2;
            au8Header[39] = u64Temp.u8_3;
            au8Header[40] = u64Temp.u8_4;
            au8Header[41] = u64Temp.u8_5;
            au8Header[42] = u64Temp.u8_6;
            au8Header[43] = u64Temp.u8_7;

            u64Temp.db = stMinPoint.Lat;
            au8Header[44] = u64Temp.u8_0;
            au8Header[45] = u64Temp.u8_1;
            au8Header[46] = u64Temp.u8_2;
            au8Header[47] = u64Temp.u8_3;
            au8Header[48] = u64Temp.u8_4;
            au8Header[49] = u64Temp.u8_5;
            au8Header[50] = u64Temp.u8_6;
            au8Header[51] = u64Temp.u8_7;

            u64Temp.db = stMaxPoint.Lng;
            au8Header[52] = u64Temp.u8_0;
            au8Header[53] = u64Temp.u8_1;
            au8Header[54] = u64Temp.u8_2;
            au8Header[55] = u64Temp.u8_3;
            au8Header[56] = u64Temp.u8_4;
            au8Header[57] = u64Temp.u8_5;
            au8Header[58] = u64Temp.u8_6;
            au8Header[59] = u64Temp.u8_7;

            u64Temp.db = stMaxPoint.Lat;
            au8Header[60] = u64Temp.u8_0;
            au8Header[61] = u64Temp.u8_1;
            au8Header[62] = u64Temp.u8_2;
            au8Header[63] = u64Temp.u8_3;
            au8Header[64] = u64Temp.u8_4;
            au8Header[65] = u64Temp.u8_5;
            au8Header[66] = u64Temp.u8_6;
            au8Header[67] = u64Temp.u8_7;

            // Record header
            DoubleUnion u32RecordNumber = new DoubleUnion();
            DoubleUnion u32RecordLength = new DoubleUnion();
            u32RecordLength.u64 = 0;
            Int32 s32FileLength = 0;
            Int32 s32EntryCount = gstRecords.s32NumberOfEntries;
            DoubleUnion u32RecordIndex = new DoubleUnion();
            u32RecordIndex.u32_0 = 100 / 2;
            u32RecordNumber.u64 = 1;
            Byte[] au8Temp = new Byte[4];

            while (s32EntryCount > 0)
            {
                u32RecordIndex.u32_0 += u32RecordLength.u32_0;
                au8Temp[0] = u32RecordIndex.u8_3;
                au8Temp[1] = u32RecordIndex.u8_2;
                au8Temp[2] = u32RecordIndex.u8_1;
                au8Temp[3] = u32RecordIndex.u8_0;
                oWriteSHX.Write(au8Temp, 0, 4);

                s32FileLength = 0;
                au8Data = new byte[10000];
                au8Data[s32FileLength++] = u32RecordNumber.u8_3;
                au8Data[s32FileLength++] = u32RecordNumber.u8_2;
                au8Data[s32FileLength++] = u32RecordNumber.u8_1;
                au8Data[s32FileLength++] = u32RecordNumber.u8_0;

                DoubleUnion u32NumberOfParts = new DoubleUnion();
                DoubleUnion u32TotalNumberOfPoint = new DoubleUnion();
                DoubleUnion[] au32PointIndex = new DoubleUnion[5];
                for (Int32 i = 0; i < 5; i++)
                {
                    au32PointIndex[i] = new DoubleUnion();
                }

                u32NumberOfParts.u32_0 = 0;
                u32TotalNumberOfPoint.u32_0 = 0;
                u32RecordLength.u32_0 = 44;

                stMinPoint = new PointLatLng(100000, 100000);
                stMaxPoint = new PointLatLng(-100000, -100000);

                for (Int32 i = 0; i < gstRecords.s32NumberOfEntries; i++)
                {
                    if (u32RecordNumber.u32_0 == gstRecords.as32RecordNumber[i])
                    {
                        s32EntryCount--;
                        u32RecordLength.u32_0 += 4;  // Length of index
                        au32PointIndex[u32NumberOfParts.u32_0].u32_0 = u32TotalNumberOfPoint.u32_0;
                        u32NumberOfParts.u32_0++;
                        u32TotalNumberOfPoint.u32_0 += (UInt32)gastPolygon[i].astPoints.Count;
                        u32RecordLength.u32_0 += (UInt32)(gastPolygon[i].astPoints.Count * 16);
                        if (gastPolygon[i].stBoundingBox.stMinPoint != null)
                        {
                            if (stMinPoint.Lng > gastPolygon[i].stBoundingBox.stMinPoint.Lng)
                                stMinPoint.Lng = gastPolygon[i].stBoundingBox.stMinPoint.Lng;
                            if (stMinPoint.Lat > gastPolygon[i].stBoundingBox.stMinPoint.Lat)
                                stMinPoint.Lat = gastPolygon[i].stBoundingBox.stMinPoint.Lat;
                            if (stMaxPoint.Lng < gastPolygon[i].stBoundingBox.stMaxPoint.Lng)
                                stMaxPoint.Lng = gastPolygon[i].stBoundingBox.stMaxPoint.Lng;
                            if (stMaxPoint.Lat < gastPolygon[i].stBoundingBox.stMaxPoint.Lat)
                                stMaxPoint.Lat = gastPolygon[i].stBoundingBox.stMaxPoint.Lat;
                        }
                    }
                }

                u32RecordLength.u32_0 /= 2;
                au8Data[s32FileLength++] = u32RecordLength.u8_3;
                au8Data[s32FileLength++] = u32RecordLength.u8_2;
                au8Data[s32FileLength++] = u32RecordLength.u8_1;
                au8Data[s32FileLength++] = u32RecordLength.u8_0;

                au8Temp[0] = u32RecordLength.u8_3;
                au8Temp[1] = u32RecordLength.u8_2;
                au8Temp[2] = u32RecordLength.u8_1;
                au8Temp[3] = u32RecordLength.u8_0;
                oWriteSHX.Write(au8Temp, 0, 4);

                u32RecordLength.u32_0 += 4; // Increment the Record length by 4, 16 bit words (record header length) so the index to the next record in the .shx file will be correct.

                au8Data[s32FileLength++] = (Byte)eShapeType.POLYGON;
                s32FileLength += 3;

                u64Temp.db = stMinPoint.Lng;
                au8Data[s32FileLength++] = u64Temp.u8_0;
                au8Data[s32FileLength++] = u64Temp.u8_1;
                au8Data[s32FileLength++] = u64Temp.u8_2;
                au8Data[s32FileLength++] = u64Temp.u8_3;
                au8Data[s32FileLength++] = u64Temp.u8_4;
                au8Data[s32FileLength++] = u64Temp.u8_5;
                au8Data[s32FileLength++] = u64Temp.u8_6;
                au8Data[s32FileLength++] = u64Temp.u8_7;

                u64Temp.db = stMinPoint.Lat;
                au8Data[s32FileLength++] = u64Temp.u8_0;
                au8Data[s32FileLength++] = u64Temp.u8_1;
                au8Data[s32FileLength++] = u64Temp.u8_2;
                au8Data[s32FileLength++] = u64Temp.u8_3;
                au8Data[s32FileLength++] = u64Temp.u8_4;
                au8Data[s32FileLength++] = u64Temp.u8_5;
                au8Data[s32FileLength++] = u64Temp.u8_6;
                au8Data[s32FileLength++] = u64Temp.u8_7;

                u64Temp.db = stMaxPoint.Lng;
                au8Data[s32FileLength++] = u64Temp.u8_0;
                au8Data[s32FileLength++] = u64Temp.u8_1;
                au8Data[s32FileLength++] = u64Temp.u8_2;
                au8Data[s32FileLength++] = u64Temp.u8_3;
                au8Data[s32FileLength++] = u64Temp.u8_4;
                au8Data[s32FileLength++] = u64Temp.u8_5;
                au8Data[s32FileLength++] = u64Temp.u8_6;
                au8Data[s32FileLength++] = u64Temp.u8_7;

                u64Temp.db = stMaxPoint.Lat;
                au8Data[s32FileLength++] = u64Temp.u8_0;
                au8Data[s32FileLength++] = u64Temp.u8_1;
                au8Data[s32FileLength++] = u64Temp.u8_2;
                au8Data[s32FileLength++] = u64Temp.u8_3;
                au8Data[s32FileLength++] = u64Temp.u8_4;
                au8Data[s32FileLength++] = u64Temp.u8_5;
                au8Data[s32FileLength++] = u64Temp.u8_6;
                au8Data[s32FileLength++] = u64Temp.u8_7;

                au8Data[s32FileLength++] = u32NumberOfParts.u8_0;
                au8Data[s32FileLength++] = u32NumberOfParts.u8_1;
                au8Data[s32FileLength++] = u32NumberOfParts.u8_2;
                au8Data[s32FileLength++] = u32NumberOfParts.u8_3;

                au8Data[s32FileLength++] = u32TotalNumberOfPoint.u8_0;
                au8Data[s32FileLength++] = u32TotalNumberOfPoint.u8_1;
                au8Data[s32FileLength++] = u32TotalNumberOfPoint.u8_2;
                au8Data[s32FileLength++] = u32TotalNumberOfPoint.u8_3;

                for (Int32 i = 0; i < u32NumberOfParts.u32_0; i++)
                {
                    au8Data[s32FileLength++] = au32PointIndex[i].u8_0;
                    au8Data[s32FileLength++] = au32PointIndex[i].u8_1;
                    au8Data[s32FileLength++] = au32PointIndex[i].u8_2;
                    au8Data[s32FileLength++] = au32PointIndex[i].u8_3;
                }

                oWriter.Write(au8Data, 0, s32FileLength);
                s32FileLength = 0;

                for (Int32 i = 0; i < gstRecords.s32NumberOfEntries; i++)
                {
                    if (u32RecordNumber.u32_0 == gstRecords.as32RecordNumber[i])
                    {
                        for (Int32 j = 0; j < gastPolygon[i].astPoints.Count; j++)
                        {
                            u64Temp.db = gastPolygon[i].astPoints[j].Lng;
                            au8Data[s32FileLength++] = u64Temp.u8_0;
                            au8Data[s32FileLength++] = u64Temp.u8_1;
                            au8Data[s32FileLength++] = u64Temp.u8_2;
                            au8Data[s32FileLength++] = u64Temp.u8_3;
                            au8Data[s32FileLength++] = u64Temp.u8_4;
                            au8Data[s32FileLength++] = u64Temp.u8_5;
                            au8Data[s32FileLength++] = u64Temp.u8_6;
                            au8Data[s32FileLength++] = u64Temp.u8_7;

                            u64Temp.db = gastPolygon[i].astPoints[j].Lat;
                            au8Data[s32FileLength++] = u64Temp.u8_0;
                            au8Data[s32FileLength++] = u64Temp.u8_1;
                            au8Data[s32FileLength++] = u64Temp.u8_2;
                            au8Data[s32FileLength++] = u64Temp.u8_3;
                            au8Data[s32FileLength++] = u64Temp.u8_4;
                            au8Data[s32FileLength++] = u64Temp.u8_5;
                            au8Data[s32FileLength++] = u64Temp.u8_6;
                            au8Data[s32FileLength++] = u64Temp.u8_7;
                            oWriter.Write(au8Data, 0, s32FileLength);
                            s32FileLength = 0;
                        }
                    }
                }
                u32RecordNumber.u32_0++;
            }

            u64Temp.u64 = ((UInt64)oOutput.Length + 100) / 2;
            au8Header[24] = u64Temp.u8_3;
            au8Header[25] = u64Temp.u8_2;
            au8Header[26] = u64Temp.u8_1;
            au8Header[27] = u64Temp.u8_0;

            oWriter.Close();
            oOutput.Close();
            oWriteSHX.Close();
            oFileSHX.Close();

            if (File.Exists(SaveFileDialog.FileName) == true)
                File.Delete(SaveFileDialog.FileName);

            oInput = new FileStream("TempFile.txt", FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite);
            oReader = new BinaryReader(oInput);

            oOutput = new FileStream(SaveFileDialog.FileName, FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite);
            oWriter = new BinaryWriter(oOutput);

            oWriter.Write(au8Header, 0, 100);

            Int32 k = 0;
            Int32 s32Count = (Int32)oInput.Length;
            au8Data = new Byte[10000];
            for (;;)
            {
                if (s32Count < 10000)
                {
                    oReader.Read(au8Data, 0, s32Count);
                    oWriter.Write(au8Data, 0, s32Count);
                    break;
                }
                else
                {
                    oReader.Read(au8Data, 0, 10000);
                    oWriter.Write(au8Data, 0, 10000);
                    s32Count -= 10000;
                    k++;
                }
            }

            oWriter.Close();
            oOutput.Close();
            oReader.Close();
            oInput.Close();

            String sPath = SaveFileDialog.FileName.Substring(0, SaveFileDialog.FileName.IndexOf('.') + 1) + "shx";
            if (File.Exists(sPath) == true)
                File.Delete(sPath);

            oInput = new FileStream("TempSHX.txt", FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite);
            oReader = new BinaryReader(oInput);

            oOutput = new FileStream(sPath, FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite);
            oWriter = new BinaryWriter(oOutput);

            u64Temp.u64 = ((UInt64)oInput.Length + 100) / 2;
            au8Header[24] = u64Temp.u8_3;
            au8Header[25] = u64Temp.u8_2;
            au8Header[26] = u64Temp.u8_1;
            au8Header[27] = u64Temp.u8_0;
            oWriter.Write(au8Header, 0, 100);

            k = 0;
            s32Count = (Int32)oInput.Length;
            au8Data = new Byte[10000];
            for (;;)
            {
                if (s32Count < 10000)
                {
                    oReader.Read(au8Data, k * 10000, s32Count);
                    oWriter.Write(au8Data, 0, s32Count);
                    break;
                }
                else
                {
                    oReader.Read(au8Data, k * 10000, 10000);
                    oWriter.Write(au8Data, 0, 10000);
                    s32Count -= 10000;
                    k++;
                }
            }

            oWriter.Close();
            oOutput.Close();
            oReader.Close();
            oInput.Close();

            s32FileLength = 0;
            au8Data[s32FileLength++] = 0x03;
            au8Data[s32FileLength++] = (Byte)(DateTime.Now.Year % 100);
            au8Data[s32FileLength++] = (Byte)(DateTime.Now.Month % 100);
            au8Data[s32FileLength++] = (Byte)(DateTime.Now.Day % 100);
            u64Temp.u32_0 = (UInt32)DBaseDataGridView.Rows.Count;
            au8Data[s32FileLength++] = u64Temp.u8_0;
            au8Data[s32FileLength++] = u64Temp.u8_1;
            au8Data[s32FileLength++] = u64Temp.u8_2;
            au8Data[s32FileLength++] = u64Temp.u8_3;

            // Calculate the index to the first record
            u64Temp.u32_0 = 0;
            for (Int32 i = 0; i < DBaseDataGridView.Columns.Count; i++)
            {
                if (DBaseDataGridView.Columns[i].HeaderText != "")
                    u64Temp.u32_0++;
            }

            Int32 s32NumberOfColumns = (Int32)u64Temp.u32_0;

            for (Int32 i = 0; i < s32NumberOfColumns; i++)
            {
                for (Int32 j = 0; j < DBaseDataGridView.Rows.Count; j++)
                {
                    if (DBaseDataGridView.Rows[j].Cells[i].Value == null)
                        DBaseDataGridView.Rows[j].Cells[i].Value = "";
                }
            }

            u64Temp.u32_0 = u64Temp.u32_0 * 32 + 33;
            au8Data[s32FileLength++] = u64Temp.u8_0;
            au8Data[s32FileLength++] = u64Temp.u8_1;

            // Calculate the size of each records data
            Int32[] as32RecordSize = new Int32[s32NumberOfColumns];
            for (Int32 i = 0; i < s32NumberOfColumns; i++)
            {
                for (Int32 j = 0; j < DBaseDataGridView.Rows.Count; j++)
                {
                    if (as32RecordSize[i] < DBaseDataGridView.Rows[j].Cells[i].Value.ToString().Length)
                        as32RecordSize[i] = DBaseDataGridView.Rows[j].Cells[i].Value.ToString().Length;
                }
            }

            u64Temp.u32_0 = 0;
            for (Int32 i = 0; i < s32NumberOfColumns; i++)
                u64Temp.u32_0 += (UInt32)as32RecordSize[i];
            au8Data[s32FileLength++] = u64Temp.u8_0;
            au8Data[s32FileLength++] = u64Temp.u8_1;

            for (Int32 i = 12; i < 32; i++)
                au8Data[s32FileLength++] = 0x00;

            // Fill in the subrecord structures
            for (Int32 i = 0; i < s32NumberOfColumns; i++)
            {
                //                au8Temp = new Byte[11];
                au8Temp = Encoding.ASCII.GetBytes(DBaseDataGridView.Columns[i].HeaderText);
                for (Int32 j = 0; j < au8Temp.Length; j++)
                    au8Data[s32FileLength++] = au8Temp[j];
                for (Int32 j = au8Temp.Length; j < 11; j++)
                    au8Data[s32FileLength++] = 0;
                au8Data[s32FileLength++] = (Byte)'C';// 0x43;
                for (Int32 j = 12; j < 16; j++)
                    au8Data[s32FileLength++] = 0;
                au8Data[s32FileLength++] = (Byte)as32RecordSize[i];
                au8Data[s32FileLength++] = 0;
                au8Data[s32FileLength++] = 2;
                for (Int32 j = 19; j < 32; j++)
                    au8Data[s32FileLength++] = 0;
            }

            au8Data[s32FileLength++] = 0x0D;

            // Fill in the record data
            for (Int32 i = 0; i < DBaseDataGridView.Rows.Count; i++)
            {
                for (Int32 j = 0; j < s32NumberOfColumns; j++)
                {
                    //                    au8Temp = new Byte[as32RecordSize[j]];
                    au8Temp = Encoding.ASCII.GetBytes(DBaseDataGridView.Rows[i].Cells[j].Value.ToString());
                    for (Int32 m = 0; m < au8Temp.Length; m++)
                        au8Data[s32FileLength++] = au8Temp[m];

                    for (Int32 m = au8Temp.Length; m < as32RecordSize[j]; m++)
                        au8Data[s32FileLength++] = (Byte)' ';
                }
            }

            au8Data[s32FileLength++] = 0x1A;

            sPath = SaveFileDialog.FileName.Substring(0, SaveFileDialog.FileName.IndexOf('.') + 1) + "dbf";
            if (File.Exists(sPath) == true)
                File.Delete(sPath);

            oOutput = new FileStream(sPath, FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite);
            oWriter = new BinaryWriter(oOutput);

            oWriter.Write(au8Data, 0, s32FileLength);

            oWriter.Close();
            oOutput.Close();
        }

        private void OpenShapefileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ShapefileOpenFileDialog.ShowDialog();

            if (File.Exists(ShapefileOpenFileDialog.FileName) == false)
                return;

            fnOpenShapeFile(ShapefileOpenFileDialog.FileName, true);
        }

        private void fnOpenShapeFile(String sFilePath, Boolean flUpdateMap)
        { 
            Byte[] au8Data = new Byte[1000];

            Stream cFile;

            UInt32 u32FileIndex = 0;

            DoubleUnion u32FileLength;
            DoubleUnion u8ShapeType;

            OpenFileDialog oFile = new OpenFileDialog();

            fnMainInit();

            StartSprayPolygonButton.Enabled = true;

            oFile.FileName = sFilePath;
            cFile = oFile.OpenFile();

            cFile.Read(au8Data, 0, 100);
            u32FileIndex += 100;

            u32FileLength.u32_0 = 0;
            u32FileLength.u8_3 = au8Data[24];
            u32FileLength.u8_2 = au8Data[25];
            u32FileLength.u8_1 = au8Data[26];
            u32FileLength.u8_0 = au8Data[27];
            u32FileLength.u32_0 *= 2;
            u8ShapeType.u8_0 = au8Data[32];

            Byte u8Index = 0;
            foreach (Byte u8Shape in gau8ShapeType)
            {
                if (u8Shape == u8ShapeType.u8_0)
                {
                    break;
                }
                u8Index++;
            }

            DoubleUnion dbTemp;
            dbTemp.db = 0;
            dbTemp.u32_0 = 0;
            DoubleUnion u32RecordNumber;
            u32RecordNumber.u32_0 = 0;
            DoubleUnion u32RecordLength;
            u32RecordLength.u32_0 = 0;
            Int32 s32PolygonIndex = 0;
            gstMinPoint = new PointLatLng(100000, 100000);
            gstMaxPoint = new PointLatLng(-100000, -100000);
            gstRecords.s32NumberOfEntries = 0;

            while (u32FileIndex < u32FileLength.u32_0)
            {
                au8Data = new Byte[4];
                cFile.Read(au8Data, 0, 4);
                u32FileIndex += 4;
                u32RecordNumber.u8_3 = au8Data[0];
                u32RecordNumber.u8_2 = au8Data[1];
                u32RecordNumber.u8_1 = au8Data[2];
                u32RecordNumber.u8_0 = au8Data[3];

                cFile.Read(au8Data, 0, 4);
                u32FileIndex += 4;
                u32RecordLength.u8_3 = au8Data[0];
                u32RecordLength.u8_2 = au8Data[1];
                u32RecordLength.u8_1 = au8Data[2];
                u32RecordLength.u8_0 = au8Data[3];
                u32RecordLength.u32_0 *= 2;

                au8Data = new Byte[u32RecordLength.u32_0];
                cFile.Read(au8Data, 0, (Int32)u32RecordLength.u32_0);
                u32FileIndex += u32RecordLength.u32_0;
                UInt32 s32FileLength = 0;

                gstRecords.aeShapeType[gstRecords.s32NumberOfEntries] = (eShapeType)au8Data[s32FileLength];

                s32FileLength += 4;
                switch (gstRecords.aeShapeType[gstRecords.s32NumberOfEntries])
                {
                    case eShapeType.NULL_SHAPE:
                        break;
                    case eShapeType.POINT:
                        break;
                    case eShapeType.MULTIPOINT:
                        break;
                    case eShapeType.POLYLINE:
                        break;
                    case eShapeType.POLYGON_Z:
                    case eShapeType.POLYGON:
                        Int32 s32NumberOfParts;
                        Int32 s32NumberOfPoints;
                        Int32[] as32PartIndex;

                        for (Byte j = 0; j < 4; j++)
                        {
                            dbTemp.u8_0 = au8Data[s32FileLength++];
                            dbTemp.u8_1 = au8Data[s32FileLength++];
                            dbTemp.u8_2 = au8Data[s32FileLength++];
                            dbTemp.u8_3 = au8Data[s32FileLength++];
                            dbTemp.u8_4 = au8Data[s32FileLength++];
                            dbTemp.u8_5 = au8Data[s32FileLength++];
                            dbTemp.u8_6 = au8Data[s32FileLength++];
                            dbTemp.u8_7 = au8Data[s32FileLength++];

                            switch(j)
                            {
                                case 0:
                                    gastPolygon[s32PolygonIndex].stBoundingBox.stMinPoint.Lng = dbTemp.db;
                                    break;
                                case 1:
                                    gastPolygon[s32PolygonIndex].stBoundingBox.stMinPoint.Lat = dbTemp.db;
                                    break;
                                case 2:
                                    gastPolygon[s32PolygonIndex].stBoundingBox.stMaxPoint.Lng = dbTemp.db;
                                    break;
                                case 3:
                                    gastPolygon[s32PolygonIndex].stBoundingBox.stMaxPoint.Lat = dbTemp.db;
                                    break;
                            }
                        }

                        dbTemp.u8_0 = au8Data[s32FileLength++];
                        dbTemp.u8_1 = au8Data[s32FileLength++];
                        dbTemp.u8_2 = au8Data[s32FileLength++];
                        dbTemp.u8_3 = au8Data[s32FileLength++];
                        s32NumberOfParts = (Int32)dbTemp.u32_0;
                        dbTemp.u8_0 = au8Data[s32FileLength++];
                        dbTemp.u8_1 = au8Data[s32FileLength++];
                        dbTemp.u8_2 = au8Data[s32FileLength++];
                        dbTemp.u8_3 = au8Data[s32FileLength++];
                        s32NumberOfPoints = (Int32)dbTemp.u32_0;

                        as32PartIndex = new Int32[s32NumberOfParts];
                        for (Int32 j = 0; j < s32NumberOfParts; j++)
                        {
                            dbTemp.u8_0 = au8Data[s32FileLength++];
                            dbTemp.u8_1 = au8Data[s32FileLength++];
                            dbTemp.u8_2 = au8Data[s32FileLength++];
                            dbTemp.u8_3 = au8Data[s32FileLength++];
                            as32PartIndex[j] = (Int32)dbTemp.u32_0;
                        }

                        for (UInt32 j = 0; j < s32NumberOfParts; j++)
                        {
                            Int32 s32Point = 0;
                            if (j + 1 == s32NumberOfParts)
                            {
                                s32Point = s32NumberOfPoints - as32PartIndex[j];
                            }
                            else
                            {
                                s32Point = as32PartIndex[j + 1] - as32PartIndex[j];
                            }

                            for (Int32 k = 0; k < s32Point; k++)
                            {
                                dbTemp.u8_0 = au8Data[s32FileLength++];
                                dbTemp.u8_1 = au8Data[s32FileLength++];
                                dbTemp.u8_2 = au8Data[s32FileLength++];
                                dbTemp.u8_3 = au8Data[s32FileLength++];
                                dbTemp.u8_4 = au8Data[s32FileLength++];
                                dbTemp.u8_5 = au8Data[s32FileLength++];
                                dbTemp.u8_6 = au8Data[s32FileLength++];
                                dbTemp.u8_7 = au8Data[s32FileLength++];

                                PointLatLng stPoint = new PointLatLng();
                                stPoint.Lng = dbTemp.db;
                                if (dbTemp.db > gstMaxPoint.Lng)
                                    gstMaxPoint.Lng = dbTemp.db;
                                if (dbTemp.db < gstMinPoint.Lng)
                                    gstMinPoint.Lng = dbTemp.db;

                                dbTemp.u8_0 = au8Data[s32FileLength++];
                                dbTemp.u8_1 = au8Data[s32FileLength++];
                                dbTemp.u8_2 = au8Data[s32FileLength++];
                                dbTemp.u8_3 = au8Data[s32FileLength++];
                                dbTemp.u8_4 = au8Data[s32FileLength++];
                                dbTemp.u8_5 = au8Data[s32FileLength++];
                                dbTemp.u8_6 = au8Data[s32FileLength++];
                                dbTemp.u8_7 = au8Data[s32FileLength++];

                                stPoint.Lat = dbTemp.db;
                                if (dbTemp.db > gstMaxPoint.Lat)
                                    gstMaxPoint.Lat = dbTemp.db;
                                if (dbTemp.db < gstMinPoint.Lat)
                                    gstMinPoint.Lat = dbTemp.db;

                                if (stPoint.Lat == 41.249529453894567)
                                    stPoint.Lat = dbTemp.db;

                                gastPolygon[s32PolygonIndex].astPoints.Add(stPoint);
                            }

                            gstRecords.as32RecordNumber[gstRecords.s32NumberOfEntries] = (int)u32RecordNumber.u32_0;
                            gstRecords.aeShapeType[gstRecords.s32NumberOfEntries] = eShapeType.POLYGON;

                            fnCalculateArea(gstRecords.s32NumberOfEntries);

                            gstRecords.s32NumberOfEntries++;

                            fnCalculateDirection(s32PolygonIndex);
                            s32PolygonIndex++;
                        }

                        List<GMapPolygon> astPolygon = new List<GMapPolygon>();
                        //for(Int32 i = 0; i < gstRecords.s32NumberOfEntries; i++)
                        //{
                        //    GMapPolygon stPolygon = new GMapPolygon(gastPolygon[i].astPoints, "");
                        //    astPolygon.Add(stPolygon);

                            
                        //}

                        //astPolygon = new List<GMapPolygon>();
                        for (Int32 i = 0; i < gstRecords.s32NumberOfEntries; i++)
                        {
                            if (gstRecords.as32RecordNumber[i] == u32RecordNumber.u32_0)
                            {
                                Int32 s32Position = 0;
                                for(Int32 j = 0; j < astPolygon.Count; j++)
                                {
                                    if ((Int32)astPolygon[j].Tag != i && astPolygon[j].IsInside(gastPolygon[i].astPoints[0]) == true)
                                    {
                                        for(Int32 k = 0; k < 5; k++)
                                        {
                                            if (gastPolygon[i].as32Position[k] <= gastPolygon[(Int32)astPolygon[j].Tag].as32Position[k])
                                                gastPolygon[i].as32Position[k] = gastPolygon[(Int32)astPolygon[j].Tag].as32Position[k];
                                            else
                                            {
                                                s32Position = k;
                                                break;
                                            }
                                        }

                                        //break;
                                    }
                                }

                                if (s32Position > 0)
                                    gastPolygon[i].as32Position[s32Position]++;
                                else
                                {
                                    for (Int32 k = 0; k < 5; k++)
                                    {
                                        if (gastPolygon[i].as32Position[k] == 0)
                                        {
                                            gastPolygon[i].as32Position[k] = 1;
                                            break;
                                        }
                                    }
                                }

                                if (astPolygon.Count == 0)
                                {
                                    gastPolygon[i].as32Position[0] = (Int32)u32RecordNumber.u32_0;
                                }

                                GMapPolygon stPolygon = new GMapPolygon(gastPolygon[i].astPoints, "");
                                stPolygon.Tag = i;
                                astPolygon.Add(stPolygon);
                            }
                        }

                        break;
                    case eShapeType.POINT_Z:
                        break;
                    case eShapeType.MULTIPOINT_Z:
                        break;
                    case eShapeType.POLYLINE_Z:
                        break;
                    case eShapeType.POINT_M:
                        break;
                    case eShapeType.MULTILINE_M:
                        break;
                    case eShapeType.POLYLINE_M:
                        break;
                    case eShapeType.POLYGON_M:
                        break;
                    case eShapeType.MULTIPATCH:
                        break;
                    default:
                        break;
                }
            }

            gs32NextRecord = (Int32)u32RecordNumber.u32_0;

            goCurrentPoint.Lat = gstMinPoint.Lat;
            goCurrentPoint.Lng = gstMinPoint.Lng;

            cFile.Close();

            if (flUpdateMap == true)
            {

                GMapControl.Zoom = 16;
                fnDrawMap(true);

                String sPath = ShapefileOpenFileDialog.FileName.Substring(0, ShapefileOpenFileDialog.FileName.IndexOf('.') + 1) + "dbf";
                if (File.Exists(sPath) == true)
                {
                    FileStream oOutput;
                    BinaryReader oReader;
                    oOutput = new FileStream(sPath, FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite);
                    oReader = new BinaryReader(oOutput);

                    au8Data = new Byte[oOutput.Length];
                    oReader.Read(au8Data, 0, (Int32)oOutput.Length);
                    DoubleUnion u32NumberOfRecords = new DoubleUnion();
                    u32NumberOfRecords.u8_0 = au8Data[4];
                    u32NumberOfRecords.u8_1 = au8Data[5];
                    u32NumberOfRecords.u8_2 = au8Data[6];
                    u32NumberOfRecords.u8_3 = au8Data[7];
                    gstDBase.u32NumberOfRecords = u32NumberOfRecords.u32_0;

                    DoubleUnion u16RecordIndex = new DoubleUnion();
                    u16RecordIndex.u8_0 = au8Data[8];
                    u16RecordIndex.u8_1 = au8Data[9];

                    u32RecordLength = new DoubleUnion();
                    u32RecordLength.u8_0 = au8Data[10];
                    u32RecordLength.u8_1 = au8Data[11];

                    Int32 s32NumberOfElements = (u16RecordIndex.u16_0 - 33) / 32;
                    gstDBase.astData = new stLABEL_NAME[s32NumberOfElements];
                    for (Byte i = 0; i < (u16RecordIndex.u16_0 - 33) / 32; i++)
                        gstDBase.astData[i].asName = new String[gstDBase.u32NumberOfRecords];

                    Byte u8ElementLength;

                    u32FileIndex = 32;
                    DBaseDataGridView.Rows.Clear();
                    DBaseDataGridView.Columns.Clear();
                    gflUpdate = false;
                    for (Byte i = 0; i < s32NumberOfElements; i++)
                    {
                        gstDBase.astData[i].sLabel = Encoding.UTF8.GetString(au8Data, (Int32)u32FileIndex, 10);

                        if (gstDBase.astData[i].sLabel.Contains('\0') == true)
                            gstDBase.astData[i].sLabel = gstDBase.astData[i].sLabel.Substring(0, gstDBase.astData[i].sLabel.IndexOf('\0'));

                        u32FileIndex += 16;
                        u8ElementLength = au8Data[u32FileIndex];
                        u32FileIndex += 16;

                        for (Int32 j = 0; j < gstDBase.u32NumberOfRecords; j++)
                            gstDBase.astData[i].asName[j] = Encoding.UTF8.GetString(au8Data, u16RecordIndex.u16_0 + j * u32RecordLength.u16_0, u8ElementLength).TrimEnd(' ').TrimStart(' ');

                        u16RecordIndex.u16_0 += u8ElementLength;

                        DBaseDataGridView.Columns.Add("Column" + i, gstDBase.astData[i].sLabel);
                        DBaseDataGridView.Columns[i].SortMode = DataGridViewColumnSortMode.NotSortable;
                    }

                    for (Byte i = 0; i < gstDBase.u32NumberOfRecords; i++)
                    {
                        DBaseDataGridView.Rows.Add();
                        for (Byte j = 0; j < s32NumberOfElements; j++)
                            DBaseDataGridView.Rows[i].Cells[j].Value = gstDBase.astData[j].asName[i];
                    }

                    gflUpdate = true;
                }

                DBaseDataGridView.Columns.Add("", "");
            }
        }

        private void fnCalculateArea(Int32 s32Record)
        {
            PointLatLng stPoint = new PointLatLng();
            PointLatLng stPolyMin = new PointLatLng();
            PointLatLng stPolyMax = new PointLatLng();
            List<PointLatLng> astPoints = new List<PointLatLng>(5000);

            Int32 s32Count = 0;

            gs16MinX = 0x7FFF;
            gs16MinY = 0x7FFF;
            gs16MaxX = 0;
            gs16MaxY = 0;

            stPolyMin.Lng = 100000;
            stPolyMin.Lat = 100000;
            stPolyMax.Lng = 0;
            stPolyMax.Lat = 0;

            for (Int32 k = 0; k < gastPolygon[s32Record].astPoints.Count; k++)
            {
                stPoint = J1939Pos2UTM(gstMinPoint.Lat, gstMinPoint.Lng, gastPolygon[s32Record].astPoints[k].Lat, gastPolygon[s32Record].astPoints[k].Lng);
                astPoints.Add(stPoint);
                if (stPoint.Lng < stPolyMin.Lng)
                    stPolyMin.Lng = stPoint.Lng;
                if (stPoint.Lng > stPolyMax.Lng)
                    stPolyMax.Lng = stPoint.Lng;
                if (stPoint.Lat < stPolyMin.Lat)
                    stPolyMin.Lat = stPoint.Lat;
                if (stPoint.Lat > stPolyMax.Lat)
                    stPolyMax.Lat = stPoint.Lat;
            }

            Int32 s32Nodes;
            Int32[] as32NodeX = new Int32[gastPolygon[s32Record].astPoints.Count - 1];
            Int32 s32PixelX;
            Int32 s32PixelY;
            Int32 o;
            Int32 m;
            Int32 s32Swap;

            //  Loop through the rows of the image.
            for (s32PixelY = (Int32)stPolyMin.Lng; s32PixelY < (Int32)stPolyMax.Lng + 1; s32PixelY++)
            {
                //  Build a list of nodes.
                s32Nodes = 0; m = gastPolygon[s32Record].astPoints.Count - 1 - 1;
                for (o = 0; o < gastPolygon[s32Record].astPoints.Count - 1; o++)
                {
                    if (astPoints[o].Lng <= (double)s32PixelY && astPoints[m].Lng >= (double)s32PixelY
                    || astPoints[m].Lng <= (double)s32PixelY && astPoints[o].Lng >= (double)s32PixelY)
                    {
                        Double dbTemp = astPoints[o].Lat + (s32PixelY - astPoints[o].Lng) / (astPoints[m].Lng - astPoints[o].Lng) * (astPoints[m].Lat - astPoints[o].Lat);
                        as32NodeX[s32Nodes++] = (Int32)(dbTemp);// + .5);
                    }
                    m = o;
                }

                //  Sort the nodes, via a simple “Bubble” sort.
                o = 0;
                while (o < s32Nodes - 1)
                {
                    if (as32NodeX[o] > as32NodeX[o + 1])
                    {
                        s32Swap = as32NodeX[o];
                        as32NodeX[o] = as32NodeX[o + 1];
                        as32NodeX[o + 1] = s32Swap;
                        if (o > 0)
                            o--;
                    }
                    else
                    {
                        o++;
                    }
                }

                //  Fill the pixels between node pairs.
                for (o = 0; o < s32Nodes; o += 2)
                {
                    if (as32NodeX[o] >= (Int32)stPolyMax.Lat) break;
                    if (as32NodeX[o + 1] > (Int32)stPolyMin.Lat)
                    {
                        if (as32NodeX[o] < (Int32)stPolyMin.Lat)
                            as32NodeX[o] = (Int32)stPolyMin.Lat;
                        if (as32NodeX[o + 1] > (Int32)stPolyMax.Lat)
                            as32NodeX[o + 1] = (Int32)stPolyMax.Lat;
                        for (s32PixelX = as32NodeX[o]; s32PixelX < as32NodeX[o + 1] + 1; s32PixelX++)
                            s32Count++;
                    }
                }
            }

            gstRecords.adbArea[s32Record] = s32Count * 0.000247105;
        }

        private void fnCalculateDirection(Int32 s32PolygonIndex)
        {
            Double dbSum = 0;
            for (Int32 i = 0; i < gastPolygon[s32PolygonIndex].astPoints.Count - 1; i++)
                dbSum += (gastPolygon[s32PolygonIndex].astPoints[i + 1].Lng - gastPolygon[s32PolygonIndex].astPoints[i].Lng) * (gastPolygon[s32PolygonIndex].astPoints[i + 1].Lat + gastPolygon[s32PolygonIndex].astPoints[i].Lat);

            dbSum += (gastPolygon[s32PolygonIndex].astPoints[0].Lng - gastPolygon[s32PolygonIndex].astPoints[gastPolygon[s32PolygonIndex].astPoints.Count - 1].Lng) * (gastPolygon[s32PolygonIndex].astPoints[0].Lat + gastPolygon[s32PolygonIndex].astPoints[gastPolygon[s32PolygonIndex].astPoints.Count - 1].Lat);

            if (dbSum > 0)
                gastPolygon[s32PolygonIndex].eDirection = eDIRECTION.CLOCKWISE;
            else
                gastPolygon[s32PolygonIndex].eDirection = eDIRECTION.COUNTER_CLOCKWISE;
        }

        private void fnDrawMap(Boolean flFill)
        {
            // Initialize map:
            switch (MapsComboBox.Text)
            {
                case "Google Satellite":
                    GMapControl.MapProvider = GMap.NET.MapProviders.GoogleSatelliteMapProvider.Instance;
                    break;
                case "Google Maps":
                    GMapControl.MapProvider = GMap.NET.MapProviders.GoogleMapProvider.Instance;
                    break;
                case "Bing Maps":
                    GMapControl.MapProvider = GMap.NET.MapProviders.BingMapProvider.Instance;
                    break;
                case "Wiki Maps":
                    GMapControl.MapProvider = GMap.NET.MapProviders.WikiMapiaMapProvider.Instance;
                    break;
                case "Open Cycle Maps":
                    GMapControl.MapProvider = GMap.NET.MapProviders.OpenCycleMapProvider.Instance;
                    break;
                case "Open Street Maps":
                    GMapControl.MapProvider = GMap.NET.MapProviders.OpenStreetMapProvider.Instance;
                    break;
                default:
                    break;
            }

            GMap.NET.GMaps.Instance.Mode = GMap.NET.AccessMode.ServerOnly;
            GMapControl.Position = goCurrentPoint;
            //            GMapControl.Position = new PointLatLng(38.98828517, -95.69392204);
            //            GMapControl.MapProvider = GMap.NET.MapProviders.GoogleSatelliteMapProvider.Instance;
            //            GMapControl.Zoom = 19;

            if (flFill == true)
            {
                goMarkersOverlay.Clear();
                goTempMarkerOverlay.Clear();
                goMainOverlay.Clear();

                List<PointLatLng> points = new List<PointLatLng>();
                points.Add(gstMinPoint);
                points.Add(new PointLatLng(gstMinPoint.Lat, gstMaxPoint.Lng));
                points.Add(gstMaxPoint);
                points.Add(new PointLatLng(gstMaxPoint.Lat, gstMinPoint.Lng));
                GMapPolygon polygon = new GMapPolygon(points, "mypolygon");

                polygon.Fill = new SolidBrush(Color.FromArgb(0, Color.White));
                polygon.Stroke = new Pen(Color.Red, 1);
                goMainOverlay.Polygons.Add(polygon);

                Int32 s32PolygonIndex = 0;
                for (Int32 j = 0; j < gstRecords.s32NumberOfEntries; j++)
                {
                    if (gstRecords.aeShapeType[j] == eShapeType.POLYGON)
                    {
                        //points = new List<PointLatLng>();
                        //for (Int32 k = 0; k < gastPolygon[j].astPoints.Count; k++)
                        //{
                        //    points.Add(gastPolygon[j].astPoints[k]);
                        //}
                        //polygon = new GMapPolygon(points, "polygon" + s32PolygonIndex);
                        polygon = new GMapPolygon(gastPolygon[j].astPoints, "polygon" + s32PolygonIndex);
                        polygon.Tag = (gstRecords.as32RecordNumber[j]).ToString();
                        polygon.IsHitTestVisible = true;
                        if (gastPolygon[j].eDirection == eDIRECTION.CLOCKWISE)
                        {
                            polygon.Fill = new SolidBrush(Color.FromArgb(40, Color.Red));
                            polygon.Stroke = new Pen(Color.Red, 1);
                        }
                        else
                        {
                            polygon.Fill = new SolidBrush(Color.FromArgb(40, Color.Black));
                            polygon.Stroke = new Pen(Color.Black, 1);
                        }
                        goMainOverlay.Polygons.Add(polygon);
                        s32PolygonIndex++;
                    }
                }
            }

            GMapControl.Overlays[0].IsVisibile = false;
            GMapControl.Overlays[0].IsVisibile = true;
        }

        private Int16 gs16MinX;
        private Int16 gs16MinY;
        private Int16 gs16MaxX;
        private Int16 gs16MaxY;

        private void fnConvertToPPDatFile()
        {
            PointLatLng stPoint = new PointLatLng();
            //PointLatLng stLastPoint = new PointLatLng();

            //Int32 s32x;
            //Int32 s32y;
            //Int32 s32Step;
            //Int32 s32XDirection;
            //Int32 s32YDirection;
            //Double dbAddStep;
            eSPRAY_STATE eSpray = eSPRAY_STATE.SPRAY;
            PointLatLng stPolyMin = new PointLatLng();
            PointLatLng stPolyMax = new PointLatLng();

            gs16MinX = 0x7FFF;
            gs16MinY = 0x7FFF;
            gs16MaxX = 0;
            gs16MaxY = 0;

            for (Int32 j = 0; j < MAXN; j++)
            {
                for (Int32 k = 0; k < MAXN; k++)
                {
                    gau8Field[j, k] = eSPRAY_STATE.OUT_OF_BOUNDS;
                }
            }

            for (Int32 j = 0; j < gstRecords.s32NumberOfEntries; j++)
            {
                stPolyMin.Lng = 100000;
                stPolyMin.Lat = 100000;
                stPolyMax.Lng = 0;
                stPolyMax.Lat = 0;

                if (gastPolygon[j].eDirection == eDIRECTION.COUNTER_CLOCKWISE)
                    eSpray = eSPRAY_STATE.NO_SPRAY;
                else
                    eSpray = eSPRAY_STATE.SPRAY;

                List<PointLatLng> astPoints = new List<PointLatLng>(5000);
                for (Int32 k = 0; k < gastPolygon[j].astPoints.Count; k++)
                {
                    stPoint = J1939Pos2UTM(gstMinPoint.Lat, gstMinPoint.Lng, gastPolygon[j].astPoints[k].Lat, gastPolygon[j].astPoints[k].Lng);
                    astPoints.Add(stPoint);
                    if (stPoint.Lng < stPolyMin.Lng)
                        stPolyMin.Lng = stPoint.Lng;
                    if (stPoint.Lng > stPolyMax.Lng)
                        stPolyMax.Lng = stPoint.Lng;
                    if (stPoint.Lat < stPolyMin.Lat)
                        stPolyMin.Lat = stPoint.Lat;
                    if (stPoint.Lat > stPolyMax.Lat)
                        stPolyMax.Lat = stPoint.Lat;
                }

                Int32 s32Nodes;
                Int32[] as32NodeX = new Int32[gastPolygon[j].astPoints.Count - 1];
                Int32 s32PixelX;
                Int32 s32PixelY;
                Int32 o;
                Int32 m;
                Int32 s32Swap;

                if (stPolyMax.Lng - stPolyMin.Lng > MAXN)
                {
                    MessageBox.Show("Cannot build a Pinpoint II map because the map is too big.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                //  Loop through the rows of the image.
                if (stPolyMin.Lng - (Int32)stPolyMin.Lng < 0.5 && stPolyMin.Lng > 0 && stPolyMin.Lng > 1)
                    stPolyMin.Lng = (Int32)(stPolyMin.Lng - 1);
                else
                    stPolyMin.Lng = (Int32)(stPolyMin.Lng);

                if (stPolyMin.Lat - (Int32)stPolyMin.Lat < 0.5 && stPolyMin.Lat > 0 && stPolyMin.Lng > 1)
                    stPolyMin.Lat = (Int32)(stPolyMin.Lat - 1);
                else
                    stPolyMin.Lat = (Int32)(stPolyMin.Lat);

                if (stPolyMax.Lng - (Int32)stPolyMax.Lng < 0.5 && stPolyMax.Lng > 1)
                    stPolyMax.Lng = (Int32)(stPolyMax.Lng);
                else
                    stPolyMax.Lng = (Int32)(stPolyMax.Lng + 1);

                if (stPolyMax.Lat - (Int32)stPolyMax.Lat < 0.5 && stPolyMax.Lat > 1)
                    stPolyMax.Lat = (Int32)(stPolyMax.Lat);
                else
                    stPolyMax.Lat = (Int32)(stPolyMax.Lat + 1);

                for (s32PixelY = (Int32)stPolyMin.Lng; s32PixelY < (Int32)stPolyMax.Lng; s32PixelY++)
                {
                    //  Build a list of nodes.
                    s32Nodes = 0; m = gastPolygon[j].astPoints.Count - 1 - 1;
                    for (o = 0; o < gastPolygon[j].astPoints.Count - 1; o++)
                    {
                        if (astPoints[o].Lng <= (double)s32PixelY && astPoints[m].Lng >= (double)s32PixelY
                        || astPoints[m].Lng < (double)s32PixelY && astPoints[o].Lng > (double)s32PixelY)
                        {
                            Double dbTemp = 0;
                            if (astPoints[m].Lng - astPoints[o].Lng == 0)
                                dbTemp = astPoints[o].Lat;
                            else
                                dbTemp = astPoints[o].Lat + (s32PixelY - astPoints[o].Lng) / (astPoints[m].Lng - astPoints[o].Lng) * (astPoints[m].Lat - astPoints[o].Lat);
                            if (dbTemp - (Int32)dbTemp > 0.5)// && dbTemp > 1)
                                dbTemp += 1;
                            as32NodeX[s32Nodes++] = (Int32)(dbTemp);// + .5);
                        }
                        m = o;
                    }

                    //  Sort the nodes, via a simple “Bubble” sort.
                    o = 0;
                    while (o < s32Nodes - 1)
                    {
                        if (as32NodeX[o] > as32NodeX[o + 1])
                        {
                            s32Swap = as32NodeX[o];
                            as32NodeX[o] = as32NodeX[o + 1];
                            as32NodeX[o + 1] = s32Swap;
                            if (o > 0)
                                o--;
                        }
                        else
                        {
                            o++;
                        }
                    }

                    //  Fill the pixels between node pairs.
                    for (o = 0; o < s32Nodes; o += 2)
                    {
                        if(as32NodeX[o + 1] - as32NodeX[o] > MAXN)
                        {
                            MessageBox.Show("Cannot build a Pinpoint II map because the map is too big.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            return;
                        }

                        if (as32NodeX[o] >= (Int32)stPolyMax.Lat) break;
                        if (as32NodeX[o + 1] > (Int32)stPolyMin.Lat)
                        {
                            if (as32NodeX[o] < (Int32)stPolyMin.Lat)
                                as32NodeX[o] = (Int32)stPolyMin.Lat;
                            if (as32NodeX[o + 1] > (Int32)stPolyMax.Lat)
                                as32NodeX[o + 1] = (Int32)stPolyMax.Lat;
                            for (s32PixelX = as32NodeX[o]; s32PixelX < as32NodeX[o + 1]; s32PixelX++)
                                gau8Field[s32PixelY, s32PixelX] = eSpray;
                        }
                    }
                }

                for (m = (Int32)stPolyMin.Lng; m < stPolyMax.Lng; m++)
                {
                    for (Int32 n = (Int32)stPolyMin.Lat; n < stPolyMax.Lat; n++)
                    {
                        if (gau8Field[m, n] == eSPRAY_STATE.NO_SPRAY)
                            gau8Field[m, n] = eSPRAY_STATE.OUT_OF_BOUNDS;
                    }
                }

                if (gs16MaxX < stPolyMax.Lng)
                    gs16MaxX = (Int16)stPolyMax.Lng;
                if (gs16MinX > stPolyMin.Lng)
                    gs16MinX = (Int16)stPolyMin.Lng;
                if (gs16MaxY < stPolyMax.Lat)
                    gs16MaxY = (Int16)stPolyMax.Lat;
                if (gs16MinY > stPolyMin.Lat)
                    gs16MinY = (Int16)stPolyMin.Lat;
            }

            gs16MaxX = (Int16)(gs16MaxX - gs16MinX);
            gs16MinX = 0;
            gs16MaxY = (Int16)(gs16MaxY - gs16MinY);
            gs16MinY = 0;

            //FileStream oInputOutput;
            //BinaryWriter oWriter;

            //oInputOutput = new FileStream("Temp.txt", FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite);
            //oWriter = new BinaryWriter(oInputOutput);

            //Byte[] au8Data = new Byte[302];
            //for (Int32 i = 0; i < 300; i++)
            //{
            //    for (Int32 j = 0; j < 300; j++)
            //    {
            //        if (gau8Field[i, j] == eSPRAY_STATE.OUT_OF_BOUNDS)
            //            au8Data[j] = Convert.ToByte('O');
            //        if( gau8Field[i,j] == eSPRAY_STATE.NO_SPRAY)
            //            au8Data[j] = Convert.ToByte('N');
            //    }
            //    au8Data[300] = Convert.ToByte('\r');
            //    au8Data[301] = Convert.ToByte('\n');
            //    oWriter.Write(au8Data, 0, (Int32)302);
            //}

            //oWriter.Close();
            //oInputOutput.Close();

        }

        /* Converts Latitude or Longitude in SAE J1939 format to degrees.  Returns degrees as a double*/
        private double J1939toDeg(double J1939FormatLatLon)
        {
            const double J1939Offset = 210.0;       //J1939 offset
            const double J1939Slope = 0.0000001;    //J1939 slope

            return ((J1939FormatLatLon * J1939Slope) - J1939Offset);
        }

        /* Converts an angle in degrees into radians.  Returns the angle in radians as a double */
        private double Deg2Rad(double Deg)
        {
            const double gPi = 3.14159265359; //Pi

            return ((gPi * Deg) / (double)180.0);
        }

        /* Converts Latitude and Longitude into Universal Trans Mercator coordinates with a base Latitude and Longitude as a (0,0)reference point.
           Returns a point with x being easting and y being northing */
        private PointLatLng J1939Pos2UTM(double Base_Latitude, double Base_Longitude, double Point_Latitude, double Point_Longitude) // Northing-Easting
        {
            const double gA = 6378137;        //Major axis of WGS-84 spheroid
            const double gE = 0.081819191;    //Eccentricity of WGS-84 ellipsoid (e = sqrt(f*(2-f))) 
            double BaseLat, PointLat, BaseLon, PointLon;
            double RadiusM, RadiusN;
            PointLatLng pt = new PointLatLng();

            BaseLat = Deg2Rad(Base_Latitude);     //Base Lat in radians
            PointLat = Deg2Rad(Point_Latitude);   //Point Lat in radians
            BaseLon = Deg2Rad(Base_Longitude);    //Base lon in radians
            PointLon = Deg2Rad(Point_Longitude);  //Point lon in radians


            RadiusM = gA * ((double)1 - gE * gE) / Math.Pow(((double)1 - gE * gE * Math.Sin(BaseLat) * Math.Sin(BaseLat)), (double)1.5);
            RadiusN = gA / Math.Pow(((double)1 - gE * gE * (Math.Sin(BaseLon)) * (Math.Sin(BaseLon))), (double)0.5);

            pt.Lng = Math.Cos(BaseLat) * RadiusN * (PointLon - BaseLon); //East-West dist. in meters
            pt.Lat = RadiusM * (PointLat - BaseLat);                //North-South dist. in meters
            return (pt);
        }

        private void fnUpdateAreaTextBox(Int32 s32Record)
        {
            Double dbAcres = 0;

            Int32 s32Index;
            for (s32Index = 1; s32Index < 5; s32Index++)
            {
                if (gastPolygon[s32Record].as32Position[s32Index] == 0)
                    break;
            }

            for (Int32 j = 0; j < gstRecords.s32NumberOfEntries; j++)
            {
                Int32 k;
                for (k = 0; k < s32Index; k++)
                {
                    if (gastPolygon[j].as32Position[k] != gastPolygon[s32Record].as32Position[k])
                        break;
                }

                if (k == s32Index)
                {
                    Int32 s32Index1;
                    for (s32Index1 = s32Index; s32Index1 < 5; s32Index1++)
                    {
                        if (gastPolygon[j].as32Position[s32Index1] == 0)
                            break;
                    }
                    s32Index1--;

                    if ((s32Index1 % 2) > 0)
                    {
                        if(s32Index1 != s32Index - 1)
                            dbAcres -= gstRecords.adbArea[j];
                    }
                    else
                        dbAcres += gstRecords.adbArea[j];
                }
            }

            SprayableAcresTextBox.Text = dbAcres.ToString();
            TotalAcresTextBox.Text = gstRecords.adbArea[s32Record].ToString();
        }

        private GMapPolygon goCurrentPolygonSelected;
        private void GMapControl_OnPolygonClick(GMapPolygon item, MouseEventArgs e)
        {
            if (gs32MarkerMoveIndex == -1 && gflEditPolygon == false)
            {
                for (Int32 i = 0; i < goMainOverlay.Polygons.Count; i++)
                {
                    if (goMainOverlay.Polygons[i] == item)
                    {
                        i--;
                        fnUpdateAreaTextBox(i);
                        break;
                    }
                }

                if (goCurrentPolygonSelected == null)
                {
                    goCurrentPolygonSelected = item;
                }
                else if (goCurrentPolygonSelected != item)
                {
                    goMarkersOverlay.Clear();
                    goCurrentPolygonSelected = item;
                }
                else
                {
                    return;
                }

                DeletePolygonButton.Enabled = true;

                GMarkerGoogle oMarker;

                goTempMarkerOverlay.Clear();
                GPSPointsDataGridView.RowCount = item.Points.Count;
                for (Int32 i = 0; i < item.Points.Count; i++)
                {
                    if (GPSPointsCheckBox.Checked == true)
                    {
                        oMarker = new GMarkerGoogle(item.Points[i], GMarkerGoogleType.green);
                        goMarkersOverlay.Markers.Add(oMarker);
                    }

                    GPSPointsDataGridView.Rows[i].Cells[(Byte)eGPS_POINT_COLUMNS.LATITUDE].Value = item.Points[i].Lat.ToString();
                    GPSPointsDataGridView.Rows[i].Cells[(Byte)eGPS_POINT_COLUMNS.LONGITUDE].Value = item.Points[i].Lng.ToString();
                }
            }
        }

        private GMapPolygon[] gaoPolygons = new GMapPolygon[5];
        private void GMapControl_OnPolygonEnter(GMapPolygon item)
        {
            for(Int32 j = 0; j < goMainOverlay.Polygons.Count; j++)
            {
                if(goMainOverlay.Polygons[j] == item && gastPolygon[j - 1].as32Position[1] == 0)
                {
                    for(Int32 k = 0; k < 5; k++)
                        gaoPolygons[k] = null;
                    break;
                }
            }


            Int32 i;
            for (i = 0; i < 5; i++)
            {
                if (gaoPolygons[i] == null || gaoPolygons[i] == item)
                {
                    if (i > 0)
                    {
                        gaoPolygons[i - 1].IsHitTestVisible = false;
                        gaoPolygons[i - 1].Stroke.Width = 1;
                    }
                    gaoPolygons[i] = item;

                    break;
                }
            }

            if (DBaseDataGridView.Rows.Count != 0)
            {
                DBaseDataGridView.Rows[Convert.ToInt32(item.Tag) - 1].Selected = true;
                DBaseDataGridView.CurrentCell = DBaseDataGridView.Rows[Convert.ToInt32(item.Tag) - 1].Cells[0];
            }

            item.Stroke.Width = 3;
        }

        private void GMapControl_OnPolygonLeave(GMapPolygon item)
        {
            if (GMapControl.MouseButtons != MouseButtons.Left)
            {
                for (Byte i = 0; i < 5; i++)
                {
                    if (gaoPolygons[i] == null)
                        break;

                    if (gaoPolygons[i] == item)
                    {
                        gaoPolygons[i] = null;
                        if (i > 0)
                        {
                            gaoPolygons[i - 1].IsHitTestVisible = true;
                            gaoPolygons[i - 1].Stroke.Width = 3;
                        }

                        break;
                    }
                }

                item.Stroke.Width = 1;
            }
        }

        private void GPSPointsClearPointsButton_Click(object sender, EventArgs e)
        {
            goCurrentPolygonSelected = null;
            for (Int32 i = 0; i < GPSPointsDataGridView.Rows.Count; i++)
            {
                GPSPointsDataGridView.Rows[i].Selected = false;
            }

            goMarkersOverlay.Clear();
            goTempMarkerOverlay.Clear();
        }

        private void GMapControl_OnMarkerEnter(GMapMarker item)
        {
            if (item.Overlay == goMarkersOverlay)
            {
                for (Int32 i = 0; i < GPSPointsDataGridView.Rows.Count; i++)
                {
                    GPSPointsDataGridView.Rows[i].Selected = false;
                }

                Int32 j = goMarkersOverlay.Markers.IndexOf(item);
                if (j >= 0 && j < GPSPointsDataGridView.Rows.Count)
                {
                    GPSPointsDataGridView.Rows[j].Selected = true;

                    goTempMarkerOverlay.Clear();
                    goTempMarkerOverlay.Markers.Add(new GMarkerGoogle(item.Position, GMarkerGoogleType.yellow));
                }
            }
        }


        private Int32 gs32MarkerMoveIndex = -1;
        private eDIRECTION geDirection = eDIRECTION.CLOCKWISE;
        private void GMapControl_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                if (gflEditPolygon == true)
                {
                    if (gastPolygon[gstRecords.s32NumberOfEntries].astPoints.Count == 0)
                    {
                        Int32 i;
                        for (i = 0; i < 5; i++)
                        {
                            if (gaoPolygons[i] == null)
                                break;
                        }

                        goMainOverlay.Polygons[gstRecords.s32NumberOfEntries].Tag = gs32NextRecord.ToString();
                        if (i > 0)
                        {
                            gstRecords.as32RecordNumber[gstRecords.s32NumberOfEntries] = Convert.ToInt32(gaoPolygons[i - 1].Tag.ToString());
                        }
                        else
                        {
                            ++gs32NextRecord;
                            gstRecords.as32RecordNumber[gstRecords.s32NumberOfEntries] = gs32NextRecord;
                        }

                        if (i % 2 == 1)
                        {
                            geDirection = eDIRECTION.COUNTER_CLOCKWISE;
                            goMainOverlay.Polygons[gstRecords.s32NumberOfEntries + 1].Fill = new SolidBrush(Color.FromArgb(40, Color.Black));
                            goMainOverlay.Polygons[gstRecords.s32NumberOfEntries + 1].Stroke = new Pen(Color.Black, 1);
                        }
                        else
                        {
                            geDirection = eDIRECTION.CLOCKWISE;
                            goMainOverlay.Polygons[gstRecords.s32NumberOfEntries + 1].Fill = new SolidBrush(Color.FromArgb(40, Color.Red));
                            goMainOverlay.Polygons[gstRecords.s32NumberOfEntries + 1].Stroke = new Pen(Color.Red, 1);
                        }
                    }

                    PointLatLng oPoint = GMapControl.FromLocalToLatLng(e.X, e.Y);
                    goMarker = new GMarkerGoogle(oPoint, GMarkerGoogleType.green);
                    goMainOverlay.Markers.Add(goMarker);

                    gastPolygon[gstRecords.s32NumberOfEntries].astPoints.Add(oPoint);

                    ++GPSPointsDataGridView.RowCount;
                    GPSPointsDataGridView.Rows[gastPolygon[gstRecords.s32NumberOfEntries].astPoints.Count - 1].Cells[(Byte)eGPS_POINT_COLUMNS.LATITUDE].Value = oPoint.Lat.ToString();
                    GPSPointsDataGridView.Rows[gastPolygon[gstRecords.s32NumberOfEntries].astPoints.Count - 1].Cells[(Byte)eGPS_POINT_COLUMNS.LONGITUDE].Value = oPoint.Lng.ToString();

                    goMainOverlay.Polygons[gstRecords.s32NumberOfEntries + 1].Points.Add(oPoint);
                }
                else
                {
                    for (Int32 i = 0; i < goMarkersOverlay.Markers.Count; i++)
                    {
                        if (goMarkersOverlay.Markers[i].IsMouseOver == true)
                        {
                            gs32MarkerMoveIndex = i;
                            break;
                        }
                    }
                }
            }
        }

        private List<PointLatLng> gaoPoints = new List<PointLatLng>();
        private GMarkerGoogle goMarker;

        private void GMapControl_MouseMove(object sender, MouseEventArgs e)
        {
            PointLatLng oPoint = GMapControl.FromLocalToLatLng(e.X, e.Y);
            if (gflEditPolygon == true)
            {
                goMainOverlay.Markers[0].Position = oPoint;
                goMainOverlay.Polygons[gstRecords.s32NumberOfEntries + 1].Points[gastPolygon[gstRecords.s32NumberOfEntries].astPoints.Count] = oPoint;
                GMapControl.UpdatePolygonLocalPosition(goMainOverlay.Polygons[gstRecords.s32NumberOfEntries + 1]);
                GPSPointsDataGridView.Rows[gastPolygon[gstRecords.s32NumberOfEntries].astPoints.Count].Cells[(Byte)eGPS_POINT_COLUMNS.LATITUDE].Value = oPoint.Lat.ToString();
                GPSPointsDataGridView.Rows[gastPolygon[gstRecords.s32NumberOfEntries].astPoints.Count].Cells[(Byte)eGPS_POINT_COLUMNS.LONGITUDE].Value = oPoint.Lng.ToString();
            }
            else if (e.Button == MouseButtons.Left && gs32MarkerMoveIndex != -1)
            {
                if (gs32MarkerMoveIndex == 0 || gs32MarkerMoveIndex == goCurrentPolygonSelected.Points.Count - 1)
                {
                    goMarkersOverlay.Markers[0].Position = oPoint;
                    goCurrentPolygonSelected.Points[0] = oPoint;
                    goMarkersOverlay.Markers[goCurrentPolygonSelected.Points.Count - 1].Position = oPoint;
                    goCurrentPolygonSelected.Points[goCurrentPolygonSelected.Points.Count - 1] = oPoint;
                    GPSPointsDataGridView.Rows[0].Cells[(Byte)eGPS_POINT_COLUMNS.LATITUDE].Value = oPoint.Lat.ToString();
                    GPSPointsDataGridView.Rows[0].Cells[(Byte)eGPS_POINT_COLUMNS.LONGITUDE].Value = oPoint.Lng.ToString();
                    GPSPointsDataGridView.Rows[goCurrentPolygonSelected.Points.Count - 1].Cells[(Byte)eGPS_POINT_COLUMNS.LATITUDE].Value = oPoint.Lat.ToString();
                    GPSPointsDataGridView.Rows[goCurrentPolygonSelected.Points.Count - 1].Cells[(Byte)eGPS_POINT_COLUMNS.LONGITUDE].Value = oPoint.Lng.ToString();

                    for (Int32 i = 1; i < goMainOverlay.Polygons.Count; i++)
                    {
                        if (goMainOverlay.Polygons[i] == goCurrentPolygonSelected)
                        {
                            gastPolygon[i - 1].astPoints = goCurrentPolygonSelected.Points;
                            fnCalculateArea(i - 1);
                            fnUpdateAreaTextBox(i - 1);
                            break;
                        }
                    }
                }
                else
                {
                    goMarkersOverlay.Markers[gs32MarkerMoveIndex].Position = oPoint;
                    goCurrentPolygonSelected.Points[gs32MarkerMoveIndex] = oPoint;
                    GPSPointsDataGridView.Rows[gs32MarkerMoveIndex].Cells[(Byte)eGPS_POINT_COLUMNS.LATITUDE].Value = oPoint.Lat.ToString();
                    GPSPointsDataGridView.Rows[gs32MarkerMoveIndex].Cells[(Byte)eGPS_POINT_COLUMNS.LONGITUDE].Value = oPoint.Lng.ToString();

                    for (Int32 i = 1; i < goMainOverlay.Polygons.Count; i++)
                    {
                        if (goMainOverlay.Polygons[i] == goCurrentPolygonSelected)
                        {
                            gastPolygon[i - 1].astPoints = goCurrentPolygonSelected.Points;
                            fnCalculateArea(i - 1);
                            fnUpdateAreaTextBox(i - 1);
                            break;
                        }
                    }
                }
                GMapControl.UpdatePolygonLocalPosition(goCurrentPolygonSelected);
                goTempMarkerOverlay.Markers[0].Position = oPoint;
            }
        }

        private void GMapControl_MouseUp(object sender, MouseEventArgs e)
        {
            if (gs32MarkerMoveIndex != -1)
            {
                gstMinPoint.Lat = 1000;
                gstMaxPoint.Lat = -1000;
                gstMinPoint.Lng = 1000;
                gstMaxPoint.Lng = -1000;
                for (Int32 i = 1; i < goMainOverlay.Polygons.Count; i++)
                {
                    if (goMainOverlay.Polygons[i] == goCurrentPolygonSelected)
                    {
                        gastPolygon[i - 1].astPoints = goCurrentPolygonSelected.Points;

                        for (Int32 j = 0; j < gstRecords.s32NumberOfEntries; j++)
                        {
                            for (Int32 k = 0; k < gastPolygon[j].astPoints.Count; k++)
                            {
                                if (gastPolygon[j].astPoints[k].Lat < gstMinPoint.Lat)
                                    gstMinPoint.Lat = gastPolygon[j].astPoints[k].Lat;
                                if (gastPolygon[j].astPoints[k].Lat > gstMaxPoint.Lat)
                                    gstMaxPoint.Lat = gastPolygon[j].astPoints[k].Lat;
                                if (gastPolygon[j].astPoints[k].Lng < gstMinPoint.Lng)
                                    gstMinPoint.Lng = gastPolygon[j].astPoints[k].Lng;
                                if (gastPolygon[j].astPoints[k].Lng > gstMaxPoint.Lng)
                                    gstMaxPoint.Lng = gastPolygon[j].astPoints[k].Lng;
                            }
                        }

//                        fnCalculateArea(i - 1);
//                        fnUpdateAreaTextBox(i - 1);
                        fnDrawMap(true);
                        break;
                    }
                }
            }
            gs32MarkerMoveIndex = -1;
        }

        private void GPSPointsDataGridView_RowEnter(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex < goMarkersOverlay.Markers.Count)
            {
                goTempMarkerOverlay.Clear();
                goTempMarkerOverlay.Markers.Add(new GMarkerGoogle(goMarkersOverlay.Markers[e.RowIndex].Position, GMarkerGoogleType.yellow));
            }
        }

        private void GPSPointsCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (goCurrentPolygonSelected != null && GPSPointsCheckBox.Checked == true)
            {
                GMarkerGoogle oMarker;

                GPSPointsDataGridView.RowCount = goCurrentPolygonSelected.Points.Count;
                for (Int32 i = 0; i < goCurrentPolygonSelected.Points.Count - 1; i++)
                {
                    if (GPSPointsCheckBox.Checked == true)
                    {
                        oMarker = new GMarkerGoogle(goCurrentPolygonSelected.Points[i], GMarkerGoogleType.green);
                        goMarkersOverlay.Markers.Add(oMarker);
                    }

                    GPSPointsDataGridView.Rows[i].Cells[(Byte)eGPS_POINT_COLUMNS.LATITUDE].Value = goCurrentPolygonSelected.Points[i].Lat.ToString();
                    GPSPointsDataGridView.Rows[i].Cells[(Byte)eGPS_POINT_COLUMNS.LONGITUDE].Value = goCurrentPolygonSelected.Points[i].Lng.ToString();
                }
            }
            else if (GPSPointsCheckBox.Checked == false)
            {
                goMarkersOverlay.Clear();
            }
        }

        private PointLatLng goCurrentPoint;
        private Boolean gflUpdate = true;
        private void MapsComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (gflUpdate == true)
            {
                if (MapsComboBox.Text == "Google Satellite" && GMapControl.Zoom < 11)
                    MapsComboBox.Text = "Google Maps";

                goCurrentPoint = GMapControl.Position;

                fnDrawMap(false);
            }
        }

        private Int32 gs32NextRecord = 0;
        private void NewMapToolStripMenuItem_Click(object sender, EventArgs e)
        {
            fnMainInit();

            goCurrentPoint = new PointLatLng(38, -95);
            GMapControl.Zoom = 3;
            fnDrawMap(false);

            StartSprayPolygonButton.Enabled = true;

            GMapPolygon oPolygon = new GMapPolygon(new List<PointLatLng>(), "polygon" + gstRecords.s32NumberOfEntries);
            goMainOverlay.Polygons.Add(oPolygon);

            DBaseDataGridView.ColumnCount = 0;
            DBaseDataGridView.RowCount = 0;
        }

        private Boolean gflEditPolygon = false;
        private void StartPointButton_Click(object sender, EventArgs e)
        {
            gaoPolygons[0] = null;

            if (StartSprayPolygonButton.Text == "Start Polygon"  && sender != null)
            {
                DeletePolygonButton.Enabled = false;
                StartSprayPolygonButton.Text = "Stop Polygon";
                gflEditPolygon = true;

                GPSPointsDataGridView.Rows.Clear();
                GPSPointsDataGridView.RowCount = 1;

                goMarker = new GMarkerGoogle(goCurrentPoint, GMarkerGoogleType.yellow);
                goMainOverlay.Markers.Add(goMarker);

                gastPolygon[gstRecords.s32NumberOfEntries].astPoints = new List<PointLatLng>();

                gstRecords.aeShapeType[gstRecords.s32NumberOfEntries] = eShapeType.POLYGON;

                GMapPolygon oPolygon = new GMapPolygon(new List<PointLatLng>(), "polygon" + gstRecords.s32NumberOfEntries);
                goMainOverlay.Polygons.Add(oPolygon);
                goMainOverlay.Polygons[gstRecords.s32NumberOfEntries + 1].Points.Add(goCurrentPoint);
            }
            else if(StartSprayPolygonButton.Text == "Stop Polygon")
            {
                StartSprayPolygonButton.Text = "Start Polygon";
                gflEditPolygon = false;

                goMainOverlay.Markers.RemoveAt(0);

                if (GPSPointsDataGridView.RowCount == 0)
                    return;

                if (GPSPointsDataGridView.RowCount > 1)
                    --GPSPointsDataGridView.RowCount;

                if (gastPolygon[gstRecords.s32NumberOfEntries].astPoints.Count < 3)
                {
                    gastPolygon[gstRecords.s32NumberOfEntries].astPoints.Clear();
                    fnDrawMap(true);
                    return;
                }

                gastPolygon[gstRecords.s32NumberOfEntries].astPoints.Add(goMainOverlay.Markers[0].Position);

                gstMinPoint.Lng = 100000;
                gstMinPoint.Lat = 100000;
                gstMaxPoint.Lng = -100000;
                gstMaxPoint.Lat = -100000;

                for (Int32 j = 0; j < gstRecords.s32NumberOfEntries + 1; j++)
                {
                    for (Int32 i = 0; i < gastPolygon[j].astPoints.Count; i++)
                    {
                        if (gastPolygon[j].astPoints[i].Lat < gstMinPoint.Lat)
                            gstMinPoint.Lat = gastPolygon[j].astPoints[i].Lat;
                        if (gastPolygon[j].astPoints[i].Lat > gstMaxPoint.Lat)
                            gstMaxPoint.Lat = gastPolygon[j].astPoints[i].Lat;
                        if (gastPolygon[j].astPoints[i].Lng < gstMinPoint.Lng)
                            gstMinPoint.Lng = gastPolygon[j].astPoints[i].Lng;
                        if (gastPolygon[j].astPoints[i].Lng > gstMaxPoint.Lng)
                            gstMaxPoint.Lng = gastPolygon[j].astPoints[i].Lng;
                    }
                }

                gastPolygon[gstRecords.s32NumberOfEntries].stBoundingBox.stMinPoint.Lng = gstMinPoint.Lng;
                gastPolygon[gstRecords.s32NumberOfEntries].stBoundingBox.stMinPoint.Lat = gstMinPoint.Lat;
                gastPolygon[gstRecords.s32NumberOfEntries].stBoundingBox.stMaxPoint.Lng = gstMaxPoint.Lng;
                gastPolygon[gstRecords.s32NumberOfEntries].stBoundingBox.stMaxPoint.Lat = gstMaxPoint.Lat;

                goCurrentPoint = GMapControl.Position;

                fnCalculateDirection(gstRecords.s32NumberOfEntries);
                if (gastPolygon[gstRecords.s32NumberOfEntries].eDirection != geDirection)
                {
                    PointLatLng oPoint;
                    for (Int32 i = 0; i < gastPolygon[gstRecords.s32NumberOfEntries].astPoints.Count / 2; i++)
                    {
                        oPoint = gastPolygon[gstRecords.s32NumberOfEntries].astPoints[i];
                        gastPolygon[gstRecords.s32NumberOfEntries].astPoints[i] = gastPolygon[gstRecords.s32NumberOfEntries].astPoints[gastPolygon[gstRecords.s32NumberOfEntries].astPoints.Count - i - 1];
                        gastPolygon[gstRecords.s32NumberOfEntries].astPoints[gastPolygon[gstRecords.s32NumberOfEntries].astPoints.Count - i - 1] = oPoint;
                    }
                    gastPolygon[gstRecords.s32NumberOfEntries].eDirection = geDirection;
                }

                ++gstRecords.s32NumberOfEntries;

                List<GMapPolygon> astPolygon = new List<GMapPolygon>();
                for (Int32 i = 0; i < gstRecords.s32NumberOfEntries - 1; i++)
                {
                    if (gstRecords.as32RecordNumber[i] == gstRecords.as32RecordNumber[gstRecords.s32NumberOfEntries - 1])
                    {
                        GMapPolygon stPolygon = new GMapPolygon(gastPolygon[i].astPoints, "");
                        stPolygon.Tag = i;
                        astPolygon.Add(stPolygon);
                        if (astPolygon[astPolygon.Count - 1].IsInside(gastPolygon[gstRecords.s32NumberOfEntries - 1].astPoints[0]) == true)
                        {
                            for (Int32 k = 0; k < 5; k++)
                            {
                                if (gastPolygon[gstRecords.s32NumberOfEntries - 1].as32Position[k] <= gastPolygon[i].as32Position[k])
                                    gastPolygon[gstRecords.s32NumberOfEntries - 1].as32Position[k] = gastPolygon[i].as32Position[k];
                                else
                                    break;

                                if (gastPolygon[gstRecords.s32NumberOfEntries - 1].as32Position[k] == 0 && k != 0)
                                {
                                //    gastPolygon[gstRecords.s32NumberOfEntries - 1].as32Position[k] = 1;
                                    break;
                                }
                            }
                        }
                    }
                }
                if (gastPolygon[gstRecords.s32NumberOfEntries - 1].as32Position[0] == 0)
                {
                    gastPolygon[gstRecords.s32NumberOfEntries - 1].as32Position[0] = gstRecords.as32RecordNumber[gstRecords.s32NumberOfEntries - 1];
                }

                Int32 s32Index = 0;
                Int32 s32LastElementIndex = 0;

                for (Int32 i = 0; i < gstRecords.s32NumberOfEntries - 1; i++)
                {
                    for (Int32 j = 0; j < 5; j++)
                    {
                        if (gastPolygon[gstRecords.s32NumberOfEntries - 1].as32Position[j] == 0)
                        {
                            s32LastElementIndex = j;
                            if (gastPolygon[i].as32Position[j] > s32Index)
                                s32Index = gastPolygon[i].as32Position[j];
                            break;
                        }

                        if (gastPolygon[gstRecords.s32NumberOfEntries - 1].as32Position[j] != gastPolygon[i].as32Position[j]) 
                        {
                            break;
                        }
                    }
                }

                if(s32LastElementIndex > 0)
                    gastPolygon[gstRecords.s32NumberOfEntries - 1].as32Position[s32LastElementIndex] = s32Index + 1;


                //for (Int32 i = 0; i < 5; i++)
                //{
                //    if (gastPolygon[gstRecords.s32NumberOfEntries - 1].as32Position[i] == 0)
                //    {
                //        for(Int32 j = 0; j < gstRecords.s32NumberOfEntries - 1; j++)
                //        {
                //            if (gastPolygon[j].as32Position[i] > s32Index)
                //                s32Index = gastPolygon[j].as32Position[i];
                //        }

                //        gastPolygon[gstRecords.s32NumberOfEntries - 1].as32Position[i] = s32Index + 1;
                //        break;
                //    }
                //}

                //Boolean flExit = false;
                //for (Int32 i = 0; i < gstRecords.s32NumberOfEntries - 1; i++)
                //{
                //    for (Int32 k = 0; k < 5; k++)
                //    {
                //        if (gastPolygon[gstRecords.s32NumberOfEntries - 1].as32Position[k] == gastPolygon[i].as32Position[k])
                //        {
                //            if (gastPolygon[gstRecords.s32NumberOfEntries - 1].as32Position[k] == 0)
                //            {
                //                ++gastPolygon[gstRecords.s32NumberOfEntries - 1].as32Position[k - 1];
                //                flExit = true;
                //                break;
                //            }
                //        }
                //        else
                //            break; 
                //    }

                //    if (flExit == true)
                //        break;
                //}

                DeletePolygonButton.Enabled = true;
                goCurrentPolygonSelected = goMainOverlay.Polygons[goMainOverlay.Polygons.Count - 1];

                fnCalculateArea(gstRecords.s32NumberOfEntries - 1);
                fnUpdateAreaTextBox(gstRecords.s32NumberOfEntries - 1);

                fnDrawMap(true);

                Int32 s32NumberOfRecords = 0;
                for (Int32 i = 0; i < gstRecords.s32NumberOfEntries; i++)
                {
                    if (gstRecords.as32RecordNumber[i] > s32NumberOfRecords)
                        s32NumberOfRecords = gstRecords.as32RecordNumber[i];
                }

                DBaseDataGridView.RowCount = s32NumberOfRecords;
            }
        }

        private void DeletePolygonButton_Click(object sender, EventArgs e)
        {
            if (goCurrentPolygonSelected != null)
            {
                Int32 i = 0;
                while (goMainOverlay.Polygons[i].Name != goCurrentPolygonSelected.Name)
                    i++;

                i--;

                Int32 s32RecordNumber = gstRecords.as32RecordNumber[i];
                Int32[] as32Positions = gastPolygon[i].as32Position;

                Byte u8NumberOfPositions;
                for (u8NumberOfPositions = 0; u8NumberOfPositions < 5; u8NumberOfPositions++)
                {
                    if (as32Positions[u8NumberOfPositions] == 0)
                        break;
                }

                GPSPointsDataGridView.Rows.Clear();

                DeletePolygonButton.Enabled = false;
                goCurrentPolygonSelected = null;

                Boolean flRemoveRecord = false;
                if (u8NumberOfPositions == 1)
                {
                    flRemoveRecord = true;
                    DBaseDataGridView.Rows.RemoveAt(gstRecords.as32RecordNumber[i] - 1);
                }

                Int32 k;
                for (Int32 j = 0; j < gstRecords.s32NumberOfEntries; j++)
                {
                    for (k = 0; k < u8NumberOfPositions; k++)
                    {
                        if (gastPolygon[j].as32Position[k] != as32Positions[k])
                            break;
                    }

                    if (k == u8NumberOfPositions)
                    {
                        gstRecords.s32NumberOfEntries--;
                        for (Int32 m = j; m < gstRecords.s32NumberOfEntries; m++)
                        {
                            gastPolygon[m] = gastPolygon[m + 1];

                            gstRecords.aeShapeType[m] = gstRecords.aeShapeType[m + 1];
                            gstRecords.as32RecordNumber[m] = gstRecords.as32RecordNumber[m + 1];
                        }

                        gstRecords.as32RecordNumber[gstRecords.s32NumberOfEntries] = 0;

                        gastPolygon[gstRecords.s32NumberOfEntries].as32Position = new Int32[5];
                        gastPolygon[gstRecords.s32NumberOfEntries].astPoints = new List<PointLatLng>();

                        j--;
                    }
                }

                for (Int32 j = 0; j < gstRecords.s32NumberOfEntries; j++)
                {
                    for (k = 0; k < u8NumberOfPositions - 1; k++)
                    {
                        if (gastPolygon[j].as32Position[k] != as32Positions[k])
                            break;
                    }

                    if(k == u8NumberOfPositions - 1)
                    {
                        if (gastPolygon[j].as32Position[u8NumberOfPositions - 1] > as32Positions[u8NumberOfPositions - 1])
                            --gastPolygon[j].as32Position[u8NumberOfPositions - 1];
                    }
                }

                if (flRemoveRecord)
                {
                    for (Int32 m = 0; m < gstRecords.s32NumberOfEntries; m++)
                    {
                        if (gstRecords.as32RecordNumber[m] > s32RecordNumber)
                        {
                            --gstRecords.as32RecordNumber[m];
//                            --gastPolygon[m].as32Position[0];
                        }
                    }
                }

                u8NumberOfPositions--;
                gs32NextRecord = 0;
                for (Int32 j = 0; j < gstRecords.s32NumberOfEntries; j++)
                {
                    if (gs32NextRecord < gastPolygon[j].as32Position[0])
                        gs32NextRecord = gastPolygon[j].as32Position[0];
                }

                gstMinPoint.Lng = 100000;
                gstMinPoint.Lat = 100000;
                gstMaxPoint.Lng = -100000;
                gstMaxPoint.Lat = -100000;

                for (Int32 j = 0; j < gstRecords.s32NumberOfEntries + 1; j++)
                {
                    for (i = 0; i < gastPolygon[j].astPoints.Count; i++)
                    {
                        if (gastPolygon[j].astPoints[i].Lat < gstMinPoint.Lat)
                            gstMinPoint.Lat = gastPolygon[j].astPoints[i].Lat;
                        if (gastPolygon[j].astPoints[i].Lat > gstMaxPoint.Lat)
                            gstMaxPoint.Lat = gastPolygon[j].astPoints[i].Lat;
                        if (gastPolygon[j].astPoints[i].Lng < gstMinPoint.Lng)
                            gstMinPoint.Lng = gastPolygon[j].astPoints[i].Lng;
                        if (gastPolygon[j].astPoints[i].Lng > gstMaxPoint.Lng)
                            gstMaxPoint.Lng = gastPolygon[j].astPoints[i].Lng;
                    }
                }

                SprayableAcresTextBox.Text = "";
                TotalAcresTextBox.Text = "";

                fnDrawMap(true);
            }
        }

        private void SaveAsShapefileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult Result = SaveFileDialog.ShowDialog();

            if (Result == DialogResult.Cancel)
                return;

            if (SaveFileDialog.FileName == "" || SaveFileDialog.FileName == null)
            {
                MessageBox.Show("Invalid File Name", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            fnConvertToShapeFile();
        }

        private void SaveAsToolPinpoint1StripMenuItem_Click(object sender, EventArgs e)
        {
        }

        private void fnSaveDataFile(String sPath)
        { 
            fnConvertToPPDatFile();

            FileStream oInputOutput;
            BinaryWriter oWriter;

            if (File.Exists(sPath))
                File.Delete(sPath);

            oInputOutput = new FileStream(sPath, FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite);
            //oInputOutput = new FileStream("Temp.txt", FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite);
            oWriter = new BinaryWriter(oInputOutput);

            Byte[] au8Temp = new Byte[MAXN * MAXN];
            UInt32 u32Temp;
            Byte k = 0;
            au8Temp[k++] = (Byte)(gs16MinX & 0x00FF);
            au8Temp[k++] = (Byte)(gs16MinX >> 8);
            au8Temp[k++] = (Byte)(gs16MinY & 0x00FF);
            au8Temp[k++] = (Byte)(gs16MinY >> 8);
            au8Temp[k++] = (Byte)(gs16MaxY & 0x00FF);
            au8Temp[k++] = (Byte)(gs16MaxY >> 8);
            au8Temp[k++] = (Byte)(gs16MaxX & 0x00FF);
            au8Temp[k++] = (Byte)(gs16MaxX >> 8);

            u32Temp = (UInt32)(gstMinPoint.Lng * 10000000 + 2100000000);
            au8Temp[k++] = (Byte)(u32Temp & 0x00FF);
            au8Temp[k++] = (Byte)((u32Temp >> 8) & 0x00FF);
            au8Temp[k++] = (Byte)((u32Temp >> 16) & 0x00FF);
            au8Temp[k++] = (Byte)(u32Temp >> 24);
            u32Temp = (UInt32)(gstMinPoint.Lat * 10000000 + 2100000000);
            au8Temp[k++] = (Byte)(u32Temp & 0x00FF);
            au8Temp[k++] = (Byte)((u32Temp >> 8) & 0x00FF);
            au8Temp[k++] = (Byte)((u32Temp >> 16) & 0x00FF);
            au8Temp[k++] = (Byte)(u32Temp >> 24);

            oWriter.Write(au8Temp, 0, k);

            //for (UInt32 i = 0; i < MAXN; i++)
            //{
            //    for (UInt32 j = 0; j < 1023; j++)
            //    {
            //        if (gau8Field[i, j] == eSPRAY_STATE.SPRAY)
            //            oWriter.Write('S');
            //        else if (gau8Field[i, j] == eSPRAY_STATE.OUT_OF_BOUNDS)
            //            oWriter.Write('O');
            //        else if (gau8Field[i, j] == eSPRAY_STATE.NO_SPRAY)
            //            oWriter.Write('N');
            //        else
            //            oWriter.Write('D');
            //    }
            //    oWriter.Write("\r\n");
            //}

            UInt32 u32FileSize = 0;
            Byte u8Count = 0;
            eSPRAY_STATE eByte = gau8Field[0, 0];
            for (UInt32 i = 0; i<MAXN; i++)
            {
                for (UInt32 j = 0; j<MAXN; j++)
                {
                    if (u8Count == 0)
                    {
                        eByte = gau8Field[i, j];
                        u8Count = 1;
                        continue;
                    }

                    if (gau8Field[i, j] != eByte || u8Count == 255)
                    {
                        au8Temp[u32FileSize++] = u8Count;
                        au8Temp[u32FileSize++] = (Byte)eByte;

                        eByte = gau8Field[i, j];
                        u8Count = 1;
                    }
                    else
                    {
                        u8Count++;
                    }
                }
            }

            if (u8Count > 0)
            {
                au8Temp[u32FileSize++] = u8Count;
                au8Temp[u32FileSize++] = (Byte)eByte;
            }

            if (u32FileSize > 0)
            {
                oWriter.Write(au8Temp, 0, (Int32)u32FileSize);
            }

            for (Byte i = 0; i< 4; i++)
            {
                oWriter.Write((Byte)((u32FileSize >> (8 * i)) & 0x000000FF));
            }

            oWriter.Close();
            oInputOutput.Close();
        }

        private void SaveAsPinpoint2MapToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MainSaveFileDialog.DefaultExt = ".dat";
            MainSaveFileDialog.Filter = "(*.dat)|*.dat";
            if (ShapefileOpenFileDialog.FileName == "" || ShapefileOpenFileDialog.FileName == null)
                MainSaveFileDialog.FileName = "Map0.dat";
            else
            {
                String sTemp = ShapefileOpenFileDialog.FileName.Substring(0, ShapefileOpenFileDialog.FileName.LastIndexOf('.'));
                MainSaveFileDialog.FileName = sTemp + ".dat";
            }

            DialogResult Result = MainSaveFileDialog.ShowDialog();
            if (Result == DialogResult.Cancel)
                return;

            if (MainSaveFileDialog.FileName == "" || MainSaveFileDialog.FileName == null)
            {
                MessageBox.Show("Invalid File Name", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            fnSaveDataFile(MainSaveFileDialog.FileName);
        }

        private void DBaseDataGridView_RowEnter(object sender, DataGridViewCellEventArgs e)
        {
            if (gflUpdate == false)
            {
                gflUpdate = true;
                return;
            }

            for (Int32 i = 1; i < gstRecords.s32NumberOfEntries + 1; i++)
                goMainOverlay.Polygons[i].Stroke.Width = 1;

            for (Int32 i = 0; i < gstRecords.s32NumberOfEntries; i++)
            {
                if (gstRecords.as32RecordNumber[i] == e.RowIndex + 1)
                {
                    goMainOverlay.Polygons[i + 1].Stroke.Width = 3;
                    goMainOverlay.Control.Invalidate(true);
                    break;
                }
            }
        }

        private void DBaseDataGridView_CellEnter(object sender, DataGridViewCellEventArgs e)
        {
            gs32ColumnNumber = e.ColumnIndex;
            if (DBaseDataGridView.Columns[e.ColumnIndex].HeaderText == "")
            {
                GMapControl.Enabled = false;
                fDBaseForm.Show();
            }
        }

        private void DBaseDataGridView_ColumnHeaderMouseClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            gs32ColumnNumber = e.ColumnIndex;
            fDBaseForm.sColumnName = DBaseDataGridView.Columns[gs32ColumnNumber].HeaderText;
            GMapControl.Enabled = false;
            fDBaseForm.Show();
        }

        private void GPSPointsDataGridView_CellLeave(object sender, DataGridViewCellEventArgs e)
        {
            PointLatLng stPoint;
            return;

            stPoint = gastPolygon[0].astPoints[e.RowIndex];
            if (e.ColumnIndex == 0)
                stPoint.Lat = Convert.ToDouble(GPSPointsDataGridView.Rows[e.RowIndex].Cells[e.ColumnIndex].Value);
            else if (e.ColumnIndex == 1)
                stPoint.Lng = Convert.ToDouble(GPSPointsDataGridView.Rows[e.RowIndex].Cells[e.ColumnIndex].Value);

            if (e.RowIndex == 0)
            {
                gastPolygon[0].astPoints.RemoveAt(gastPolygon[0].astPoints.Count - 1);
                gastPolygon[0].astPoints.Insert(gastPolygon[0].astPoints.Count - 1, stPoint);
            }

            gastPolygon[0].astPoints.RemoveAt(e.RowIndex);
            gastPolygon[0].astPoints.Insert(e.RowIndex, stPoint);
        }

        private void PlusButton_Click(object sender, EventArgs e)
        {
            GMapControl.Zoom++;
        }

        private void MinusButton_Click(object sender, EventArgs e)
        {
            GMapControl.Zoom--;
        }

        private void BatchPinpoint1MapConversionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            BatchOpenFileDialog.ShowDialog();
            String[] asFiles = BatchOpenFileDialog.FileNames;

            if (asFiles.Length > 0)
            {
                for(Int32 i = 0; i < asFiles.Length; i++)
                {
                    fnOpenShapeFile(asFiles[i], false);

                    String sTemp = asFiles[i].Substring(0, asFiles[i].LastIndexOf('.'));
                    fnSaveDataFile(sTemp + ".dat");
                }
            }
        }

        private void GMapControl_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            StartPointButton_Click(null, null);
        }

        private void versionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Version 1.00.16", "", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void manualToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Start Internet Explorer. Defaults to CapstanAG.com/CapMaps.
            Process.Start("IExplore.exe", "CapstanAG.com/CapMaps");
        }
    }

    public enum eShapeType
    {
        NULL_SHAPE = 0,
        POINT = 1,
        POLYLINE = 3,
        POLYGON = 5,
        MULTIPOINT = 8,
        POINT_Z = 11,
        POLYLINE_Z = 13,
        POLYGON_Z = 15,
        MULTIPOINT_Z = 18,
        POINT_M = 21,
        POLYLINE_M = 23,
        POLYGON_M = 25,
        MULTILINE_M = 28,
        MULTIPATCH = 31
    }

    public enum eGPS_POINT_COLUMNS
    {
        LATITUDE = 0,
        LONGITUDE
    }

    public enum eSPRAY_STATE
    {
        OUT_OF_BOUNDS = 0,
        SPRAY = 50,
        NO_SPRAY = 200,
    }

    public enum eDIRECTION
    {
        CLOCKWISE,
        COUNTER_CLOCKWISE
    }

    public struct stLABEL_NAME
    {
        public String sLabel;
        public String[] asName;
    }

    public struct stDBASE_DATA
    {
        public UInt32 u32NumberOfRecords;
        public stLABEL_NAME[] astData;
    }

    public struct stRECORDS
    {
        public Int32 s32NumberOfEntries;
        public eShapeType[] aeShapeType;
        public Int32[] as32RecordNumber;
        public Double[] adbArea;
    }

    public struct stPOLYGON
    {
        public eDIRECTION eDirection;
        public stBOUNDING_BOX stBoundingBox;
        public List<PointLatLng> astPoints;
        public Int32[] as32Position;
    }

    public struct stBOUNDING_BOX
    {
        public PointLatLng stMinPoint;
        public PointLatLng stMaxPoint;
    }

    [System.Runtime.InteropServices.StructLayout(LayoutKind.Explicit)]
    public struct DoubleUnion
    {
        [System.Runtime.InteropServices.FieldOffset(0)]
        public Byte u8_0;
        [System.Runtime.InteropServices.FieldOffset(1)]
        public Byte u8_1;
        [System.Runtime.InteropServices.FieldOffset(2)]
        public Byte u8_2;
        [System.Runtime.InteropServices.FieldOffset(3)]
        public Byte u8_3;
        [System.Runtime.InteropServices.FieldOffset(4)]
        public Byte u8_4;
        [System.Runtime.InteropServices.FieldOffset(5)]
        public Byte u8_5;
        [System.Runtime.InteropServices.FieldOffset(6)]
        public Byte u8_6;
        [System.Runtime.InteropServices.FieldOffset(7)]
        public Byte u8_7;
        [System.Runtime.InteropServices.FieldOffset(0)]
        public UInt16 u16_0;
        [System.Runtime.InteropServices.FieldOffset(2)]
        public UInt16 u16_1;
        [System.Runtime.InteropServices.FieldOffset(4)]
        public UInt16 u16_2;
        [System.Runtime.InteropServices.FieldOffset(6)]
        public UInt16 u16_3;
        [System.Runtime.InteropServices.FieldOffset(0)]
        public UInt32 u32_0;
        [System.Runtime.InteropServices.FieldOffset(4)]
        public UInt32 u32_1;
        [System.Runtime.InteropServices.FieldOffset(0)]
        public UInt64 u64;
        [System.Runtime.InteropServices.FieldOffset(0)]
        public Double db;
    }
}
